<?php
/*
Original code by Skakunov Alexander <i1t2b3@gmail.com> (Quick_CSV_import)
Modify by Daniele Ugoletti for MWCMS
*/


class CVSimporter
{
  var $table_name; //where to import to
  var $file_name;  //where to import from
  var $use_csv_header; //use first line of file OR generated columns names
  var $add_header_prefix; //use first line of file OR generated columns names
  var $field_separate_char; //character to separate fields
  var $field_enclose_char; //character to enclose fields, which contain separator char into content
  var $field_escape_char;  //char to escape special symbols
  var $error; //error message
  var $arr_csv_columns; //array of columns
  var $table_exists; //flag: does table for import exist
  var $encoding; //encoding table, used to parse the incoming file. Added in 1.5 version
  
  var $conn;
  var $dropTable;

  function CVSimporter( $file_name="" )
  {
    $this->file_name = $file_name;
    $this->arr_csv_columns = array();
    $this->use_csv_header = true;
    $this->add_header_prefix = true;
    $this->field_separate_char = ",";
    $this->field_enclose_char  = "\"";
    $this->field_escape_char   = "\\";
    $this->table_exists = false;
	$this->dropTable = false;
  }
  
  function import( $isUpdate = false, $fields = null )
  {
    if($this->table_name=="")
      $this->table_name = "temp_".date("d_m_Y_H_i_s");
    
    if ( !$isUpdate )
    {
	    $this->table_exists = false;
	    $this->create_import_table();
    }
	else
	{
		$this->table_exists = true;
		if ( $this->dropTable )
		{
			$sql = "TRUNCATE TABLE `".$this->table_name."`;";
			$res = $this->conn->execute($sql);
		}
	}   
	 
    if(empty($this->arr_csv_columns))
      $this->get_csv_header_fields();
    
    if ( is_array( $fields ) )
    {
    	$newFields = array();
    	for( $i=0; $i<count( $this->arr_csv_columns ); $i++ )
    	{
    		$newFields[] = $i < count( $fields ) ? $fields[ $i ] : $this->arr_csv_columns[ $i ];
    	}

    	$this->arr_csv_columns = $newFields;
    }

    if($this->table_exists)
    {
		if  ( ( $handle = fopen($this->file_name, "r" ) ) !== FALSE )
		{
			$row = 0;
			$sql = "INSERT INTO `".$this->table_name."` (`".implode("`,`", $this->arr_csv_columns)."`) VALUES ";
			$sqlPart = array();
		    while (($data = fgetcsv($handle, 0,  $this->field_separate_char, $this->field_enclose_char )) !== FALSE)
			{
				$row++;
				if ( $this->use_csv_header && $row == 1 ) continue;
				$num = count($data);
				$sqlTemp = "(";
				for ($c=0; $c < $num; $c++)
				{
					$v = str_replace( $this->field_escape_char.$this->field_enclose_char, $this->field_enclose_char, $data[$c] );
					$sqlTemp .= $this->conn->qstr( $this->encoding == "utf8" ? utf8_decode( $v ) : $v, false );
					if ( $c != $num-1 )
					{
						$sqlTemp .= ", ";
					}
				}
				$sqlTemp .= ")";
				$sqlPart[] = $sqlTemp;
		    }
			$sql .= implode( ", ", $sqlPart ).";";
		    fclose($handle);
		
			$res = $this->conn->execute($sql);
			$this->error = $this->conn->ErrorMsg();
		
			if ( !$isUpdate )
			{			
				$sql = "ALTER TABLE `".$this->table_name."` CHANGE `".$this->table_name."_id` int(10) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY";
				$res = $this->conn->execute($sql, array(), true );
				$sql = "ALTER TABLE `".$this->table_name."` ADD `".$this->table_name."_id` int(10) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY";
				$res = $this->conn->execute($sql, array(), true );
			}
		}
		else
		{
			$this->error = "file cannot be opened: ".(""==$this->file_name ? "[empty]" : $this->file_name );
		}
    }
  }
  
  //returns array of CSV file columns
  function get_csv_header_fields()
  {

	if  ( ( $handle = fopen($this->file_name, "r" ) ) !== FALSE )
	{
		$data = fgetcsv($handle, 0,  $this->field_separate_char, $this->field_enclose_char );
		$num = count($data);
		for ($c=0; $c < $num; $c++)
		{
			if($this->use_csv_header && !empty($data[$c]) )
	        {
				$colName = $data[$c];
				$colName = strtolower($colName);
				$colName = preg_replace('/&.+?;/', '', $colName);
				$colName = preg_replace('/[^a-z0-9 _-]/', '', $colName);
				$colName = preg_replace('/\s+/', ' ', $colName);
				$colName = str_replace(' ', '_', $colName);
				$colName = preg_replace('|-+|', '_', $colName);
            	$this->arr_csv_columns[] = $this->add_header_prefix ? "column_".$colName : $colName;
			}
			else
			{
				$this->arr_csv_columns[] = "column".$c++;
			}
		
        }
	    fclose($handle);
	}
	else
	{
		$this->error = "file cannot be opened: ".(""==$this->file_name ? "[empty]" : $this->file_name );
	}
	
    return $this->arr_csv_columns;
  }
  
  function create_import_table()
  {
	$sql = "CREATE TABLE IF NOT EXISTS ".$this->table_name." (";
    
    if(empty($this->arr_csv_columns))
      $this->get_csv_header_fields();
    
    if(!empty($this->arr_csv_columns))
    {
      $arr = array();
      for($i=0; $i<sizeof($this->arr_csv_columns); $i++)
          $arr[] = "`".$this->arr_csv_columns[$i]."` TEXT";
      $sql .= implode(",", $arr);
      $sql .= ")";
      $res = $this->conn->execute($sql);
      $this->error = $this->conn->ErrorMsg();
      $this->table_exists = $this->error == "";
    }
  }

  //defines the encoding of the server to parse to file
  function set_encoding($encoding="")
  {
	$this->encoding = $encoding;
    // if("" == $encoding)
    //     $encoding = $this->encoding;
    //   $sql = "SET SESSION character_set_database = " . $encoding; //'character_set_database' MySQL server variable is [also] to parse file with rigth encoding
    //   $res = $this->conn->execute($sql);
    //   return$this->conn->ErrorMsg();
  }
}

?>