<?php
/* SVN FILE: $Id$ */

/**
 * 
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.application
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */

glz_import('org.glizy.helpers.Link');

if (!defined('GLZ_REQUEST_ALL')) define('GLZ_REQUEST_ALL', 0);
if (!defined('GLZ_REQUEST_GET')) define('GLZ_REQUEST_GET', 1);
if (!defined('GLZ_REQUEST_POST')) define('GLZ_REQUEST_POST', 2);
if (!defined('GLZ_REQUEST_VALUE')) define('GLZ_REQUEST_VALUE', 0);
if (!defined('GLZ_REQUEST_TYPE')) define('GLZ_REQUEST_TYPE', 1);

class org_glizy_Request
{
	static $decodeUtf8 = false;
	static $skipDecode = false;
	static $translateInfo = true;
	
	function org_glizy_Request()
	{
	}

	function init()
	{
		$params	= &org_glizy_Request::_getValuesArray(true);
		$charset = strtolower( org_glizy_Config::get('CHARSET') );
		$requestCharset = @$_SERVER[ 'CONTENT_TYPE' ];
		if ( $charset != "utf-8" && stripos( $requestCharset, 'utf-8' ) !== false )
		{
			self::$decodeUtf8 = true;
		}
		
		if ( self::$skipDecode )
		{
			self::$decodeUtf8 = false;
		}

		foreach($_GET as $k=>$v)
		{
			if (!is_array($v) && get_magic_quotes_gpc()) $v = stripslashes($v);
			if ( self::$decodeUtf8 ) $v = org_glizy_Request::utf8_decode($v);
			$url .= '&'.$k.'='.$v;
			$params[$k]= array($v, GLZ_REQUEST_GET);
		}

		foreach($_POST as $k=>$v)
		{
			if (!is_array($v) && get_magic_quotes_gpc()) $v = stripslashes($v);
			if ( self::$decodeUtf8 ) $v = org_glizy_Request::utf8_decode($v);
			$url .= '&'.$k.'='.$v;
			$params[$k]= array($v, GLZ_REQUEST_POST);
		}
	
		$params[ '__url__' ] = array( __Routing::$requestUrl, GLZ_REQUEST_GET );
		$params[ '__back__url__' ] = array( $url, GLZ_REQUEST_GET );
		

		if ( self::$translateInfo )
		{
			$pageId = strtolower( $params[ 'pageId' ][ GLZ_REQUEST_VALUE ] );
			$translateInfo = __Session::get( '__translateInfo_'.$pageId, array( ) );
			foreach( $translateInfo as $v )
			{
				if ( $params[ $v[ 'target' ] ][ GLZ_REQUEST_VALUE ] == $v[ 'label' ] )
				{
					$params[ $v[ 'target' ] ][ GLZ_REQUEST_VALUE ] =  $v[ 'value' ];
				}
			}
			__Session::remove( '__translateInfo_'.$pageId );
		}
	}
	
	function get($name, $defaultValue=NULL, $type=GLZ_REQUEST_ALL)
	{
		$params	= &org_glizy_Request::_getValuesArray();
		$value = array_key_exists($name, $params) ? $params[$name] : NULL;
		$value = is_null($value) ? $defaultValue : ($type==GLZ_REQUEST_ALL || $type==$value[GLZ_REQUEST_TYPE] ? $value[GLZ_REQUEST_VALUE] : $defaultValue);
		return $value;
	}
	
	function set($name, $value, $type=GLZ_REQUEST_ALL)
	{
		$params	= &org_glizy_Request::_getValuesArray();
		
		$params[$name] = array($value, $type);
	}
	
	function add($name, $value, $type=GLZ_REQUEST_ALL)
	{
		$params	= &org_glizy_Request::_getValuesArray();		
		if (isset($params[$name]))
		{
			trigger_error('The param is already set');
		}
		
		$params[$name] = array($value, $type);
	}

	function exists($key)
	{
		$params	= &org_glizy_Request::_getValuesArray();
		return isset($params[$key]);
	}
	
	function isEqual( $name, $value )
	{
		return strtolower( org_glizy_Request::get( $name, '' ) ) == strtolower( $value );
	}
	
	function remove($name)
	{
		$params	= &org_glizy_Request::_getValuesArray();
		if (isset($params[$name]))
		{
			unset($params[$name]);
		}
	}
	
	function removeAll()
	{
		$params	= &org_glizy_Request::_getValuesArray();
		$params = array();
	}
		
	function getAllAsArray()
	{
		$params	= &org_glizy_Request::_getValuesArray();
		$result = array();
		foreach($params	as $k=>$v)
		{
			$result[$k] = $v[GLZ_REQUEST_VALUE];
		}
		return $result;
	}

	function setFromArray($values)
	{
		$params	= &org_glizy_Request::_getValuesArray();
		foreach($values as $k=>$v)
		{
			$params[$k] = is_array($v) ? $v : array($v, GLZ_REQUEST_ALL);
		}
	}
	
	function dump()
	{
		$params	= &org_glizy_Request::_getValuesArray();
		var_dump($params);
	}
	
	function &_getValuesArray($init=false)
	{
		static $_valuesArray = array();		
		if (!count($_valuesArray) && !$init)
		{
			org_glizy_Request::init();
		}
		return $_valuesArray;
	}
	
	function utf8_decode($values)
	{
		if (is_array($values))
		{
			$keys = array_keys($values);
			$count = count($values);
			for ($i = 0; $i < $count; $i++)
			{
				if (is_array($values[$keys[$i]]))
				{
					$values[$keys[$i]] = org_glizy_Request::utf8_decode($values[$keys[$i]]);
				}
				else
				{
					if ( function_exists('iconv') )
					{
						$output[$keys[$i]] = iconv("UTF-8", "CP1252", $values[$keys[$i]]);
					}
					else
					{
						$output[$keys[$i]] = utf8_decode($values[$keys[$i]]);
					}
				}
			}
			return $values;
		}
		else
		{
			if( function_exists('iconv') )
			{
				return iconv( "UTF-8", "CP1252", $values );
			}
			else
			{
				return utf8_decode( $values );
			}
		}
	}
}

// shortchut version
class __Request extends org_glizy_Request
{
}
?>