<?php
/* SVN FILE: $Id$ */

/**
 * 
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.admin.components
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 admin component
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */
 
glz_import('org.glizy.admin.components.JSEditing');

class org_glizy_admin_components_FiltersFromModel extends org_glizy_admin_components_JSEditing
{
	var $_filters;
	var $_filtersJs;
	var $_sessionEx;
	
	/**
	 * Return the component information
	 * 
	 * @return	array	component informaton.
	 * @access	public
	 * @static
	 */
	function getInfo()
	{
		$info 					= parent::info();
		$info['name']			= 'FiltersFromModel';
		$info['class'] 			= 'org.glizy.components.FiltersFromModel';
		$info['package'] 		= 'Glizy';
		$info['version'] 		= GLZ_CORE_VERSION;
		$info['author'] 		= 'Daniele Ugoletti';
		$info['author-email'] 	= 'daniele@ugoletti.com';
		$info['url'] 			= 'http://www.ugoletti.com';
		return $info;
	}
	
	/**
	 * Init
	 * 
	 * @return	void
	 * @access	public
	 */
	function init()
	{
		// define the custom attributes
		$this->defineAttribute('label',				false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('recordClassName',	true, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('target',			true, 	NULL,	COMPONENT_TYPE_OBJECT);	

		// call the superclass for validate the attributes
		parent::init();
		$this->doLater($this, 'readFilters');
	}
	

	function process()
	{	
		// crea i figli 
		$className = $this->getAttribute('recordClassName');
		$this->createChildsFromModel($className);
		$this->processChilds();
	}


	/**
	 * Render
	 * 
	 * @return	void
	 * @access	public
	 */
	function render()
	{
		$this->renderChilds('jsediting');

		// creal l'array per l'ouput dei dati.
		$output = array();
		$output['id'] = $this->getId();
		$output['iconPath'] = org_glizy_Paths::get('CORE_STATIC_DIR').'assets/images/';
		$output[] = array('id' => 'items');
		$output[] = array('id' => 'values', 'values' => $this->_filtersJs);

		foreach ($this->_output as $filter)
		{
			if ($filter['type']!='hidden' && $filter['type']!='image' && $filter['type']!='static' && $filter['type']!='repeater')
			{
				unset($filter['value']);
				unset($filter['required']);
				unset($filter['requiredMessage']);
				unset($filter['append']);
				$filter['id'] = 'items.'.preg_replace('/([^\-]+)\-(.*)/', '$2', $filter['id']);
				$output[$filter['id']] = $filter;
			}
		}
		
		$jsOuput  = '<script type="text/javascript">'.GLZ_COMPILER_NEWLINE2;
		$this->serializeJs($output, $jsOuput, 'Glizy.pageFilters', true);
		$jsOuput .= '</script>'.GLZ_COMPILER_NEWLINE2;
		$this->addParentOutputCode($jsOuput);
		
		if (!is_null($this->getAttribute('label')))
		{
			$output = '<p class="genericCaption">'.$this->getAttribute('label').'</p>';	
			$this->addParentOutputCode($output);
		}
		
		$this->addParentOutputCode(org_glizy_helpers_JS::linkCoreJSfile('Glizy_pageFilters.js?v='.GLZ_CORE_VERSION));
	}
	
	function loadComponentContent()
	{
		return '';	
	}

	function getFilters()
	{
		return $this->_filters;
	}
	
	function readFilters()
	{	
		$this->_sessionEx	= org_glizy_ObjectFactory::createObject('org.glizy.SessionEx', $this->getId());
		$target = $this->getAttribute('target');
		$command = org_glizy_Request::get($target->getId().'_command', NULL);
		
		// controlla se sono stati definiti i filtri
		$this->_filters = array();
		$this->_filtersJs = array();
		
		if ($command=='SEARCH')
		{
			$filterCount = org_glizy_Request::get($this->getId().'_count', 0);
			for ($i=0; $i<=$filterCount;$i++)
			{
				if (org_glizy_Request::get($this->getId().'_type_'.$i, '')!='') 
				{
					$this->_filters[org_glizy_Request::get($this->getId().'_type_'.$i, '')] = org_glizy_Request::get($this->getId().'_value_'.$i, '');
					$this->_filtersJs[] = array(org_glizy_Request::get($this->getId().'_type_'.$i, ''), org_glizy_Request::get($this->getId().'_value_'.$i, ''));
				}
			}
		}
		else if ($command!='RESET')
		{
			$filters = $this->_sessionEx->get('filters', NULL);
			$filtersJs = $this->_sessionEx->get('filtersJs', NULL);
			if (!is_null($filters)) $this->_filters = $filters;
			if (!is_null($filtersJs)) $this->_filtersJs = $filtersJs;
		}
		
		$this->_sessionEx->set('filters', $this->_filters);
		$this->_sessionEx->set('filtersJs', $this->_filtersJs);
	}
}
?>