<?php
/* SVN FILE: $Id$ */

/**
 * Application  class.
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.application
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */


glz_import('org.glizy.locale.Locale');
glz_import('org.glizy.application.User');
glz_import('org.glizy.helpers.JS');
glz_import('org.glizy.helpers.CSS');
glz_import('org.glizy.log.LogBase');
glz_import('org.glizy.log.LogFactory');
glz_import('org.glizy.application.MessageStack');


class org_glizy_application_Application extends GlizyObject
{
	var $_pathCore				= '';
	var $_pathApplication		= '';
	var $_pageId				= 0;
	var $_pageType				= '';
	var $outputMode				= 'html';
	var $siteMap				= NULL;
	var $siteMapMenu			= NULL;
	var $_rootComponent			= NULL;
	var $_user					= NULL;
	var $_language				= '';
	var $_languageId			= 0;
	var $_ajaxMode				= false;
	var $_logObj				=	 NULL;

	function org_glizy_application_Application($pathApplication='', $pathCore='')
	{
		if (empty($pathApplication))
		{
			new org_glizy_Exception(array('[%s] %s', $this->getClassName(), GLZ_ERR_EMPTY_APP_PATH));
		}

		org_glizy_ObjectValues::setByReference('org.glizy', 'application', $this);
		$this->_pathApplication = $pathApplication;
		$this->_pathCore 		= $pathCore;
		$this->addEventListener(GLZ_EVT_USERLOGIN, $this);	
		$this->_init();
	}
	
	function getInfo()
	{
		$info 					= parent::info();
		$info['name']			= 'Application';
		$info['class'] 			= 'org.glizy.application.Application';
		$info['package'] 		= 'Glizy';
		$info['version'] 		= ''; // GLZ_CORE_VERSION;
		$info['author'] 		= 'Daniele Ugoletti';
		$info['author-email'] 	= 'daniele@ugoletti.com';
		$info['url'] 			= 'http://www.glizy.org';
		return $info;
	}
	

	function run()
	{
		$this->log( "Run application", GLZ_LOG_SYSTEM );
		if (file_exists(org_glizy_Paths::get('APPLICATION_STARTUP')))
		{
			// if the startup folder is defined all files are included
			glz_require_once_dir(org_glizy_Paths::get('APPLICATION_STARTUP'));
		}

		$this->_initSiteMap();
		$this->_initRequest();
		
		glz_require_once_dir(org_glizy_Paths::getRealPath('APPLICATION_CLASSES'));
		
		$this->_startProcess();

		if (file_exists(org_glizy_Paths::get('APPLICATION_SHUTDOWN')))
		{
			// if the shutdown folder is defined all files are included
			glz_require_once_dir(org_glizy_Paths::get('APPLICATION_SHUTDOWN'));
		}
	}

	function runAjax()
	{
		$this->log( "Run ajax application", GLZ_LOG_SYSTEM );
		org_glizy_Request::$translateInfo = false;
		// org_glizy_Config::set('CHARSET', 'utf-8' );
		//org_glizy_Request::$decodeUtf8 = strtolower( org_glizy_Config::get('CHARSET') ) != "utf-8";
		org_glizy_Request::$skipDecode = true;
		
		$this->_ajaxMode = true;
		$this->run();
	}
	
	function runSoft()
	{
		$this->log( "Run application (soft mode)", GLZ_LOG_SYSTEM );
		
		if (file_exists(org_glizy_Paths::get('APPLICATION_STARTUP')))
		{
			// if the startup folder is defined all files are included
			glz_require_once_dir(org_glizy_Paths::get('APPLICATION_STARTUP'));
		}
		
		$this->_initSiteMap();
		$this->_initRequest();
		
		if (file_exists(org_glizy_Paths::get('APPLICATION_SHUTDOWN')))
		{
			// if the shutdown folder is defined all files are included
			glz_require_once_dir(org_glizy_Paths::get('APPLICATION_SHUTDOWN'));
		}
	}
	
	function _init()
	{
		// inizializzazione delle classi
		// classe statica per la gestione dei path
		org_glizy_Paths::init($this->_pathApplication, $this->_pathCore);
		
		// legge i parametri di configurazione
		org_glizy_Config::init();
	
		
		$sessionPrefix = org_glizy_Config::get('SESSION_PREFIX');
		if (empty($sessionPrefix))
		{
			// se non  stato specificato un prefisso per la sessione
			// viene usato il nome dell'applicazione
			org_glizy_Config::set('SESSION_PREFIX', str_replace(array('.', ' ', '/'), '', $this->_pathApplication).'_');
		}

		// inizializzazione della sessione
		__Session::start();
		if (org_glizy_config::get('LOG_FILE')!='')
		{
			if ( org_glizy_config::get('LOG_FILE') != "firebug" )
			{
			$this->_logObj = org_glizy_log_LogFactory::create('File', org_glizy_Paths::get('APPLICATION').'/'.org_glizy_config::get('LOG_FILE'), array(), org_glizy_config::get('LOG_LEVEL'));
			}
			else
			{
				$this->_logObj = org_glizy_log_LogFactory::create('FireBug', array(), org_glizy_config::get('LOG_LEVEL'));
			}
			$this->log( "Start application", GLZ_LOG_SYSTEM );
		}
		
		$this->login();
		$this->_initLanguage();
	}

	function _initRequest()
	{
		org_glizy_Routing::init();
		org_glizy_Routing::_parseUrl();
		org_glizy_Request::init();
	}
	
	
	function _initLanguage()
	{
		$this->log( "initLanguage", GLZ_LOG_SYSTEM );
		// inizializza la lingua
		$this->_language 	= org_glizy_Config::get('DEFAULT_LANGUAGE');
		$this->_languageId 	= org_glizy_Config::get('DEFAULT_LANGUAGE_ID');
		$this->_loadLocale();
	}

	function _initSiteMap()
	{
		$this->log( "initSiteMap", GLZ_LOG_SYSTEM );
		$this->siteMap = &org_glizy_ObjectFactory::createObject('org.glizy.application.SiteMapSimple');
		$this->siteMap->getSiteArray();
	}

	function _readPageId()
	{
		$this->log( "readPageId", GLZ_LOG_SYSTEM );
		// legge il pageId della pagina da visualizzare
		$this->_pageId = org_glizy_Request::get('pageId', NULL);
		if (empty($this->_pageId))
		{
			$this->_pageId =  org_glizy_Config::get('REMEMBER_PAGEID') ? org_glizy_Session::get('glizy.pageId', org_glizy_Config::get('START_PAGE')) : org_glizy_Config::get('START_PAGE');
		}
		
		if (org_glizy_Config::get('REMEMBER_PAGEID')) 
		{
			org_glizy_Session::set('glizy.pageId', $this->_pageId);
		}
		
		if (!is_numeric($this->_pageId) && ( $this->getClassName()=='org_glizy_application_applicationdb' || $this->getClassName()=='org_glizy_mvc_core_applicationdb') )
		{
			$this->siteMapMenu	= &$this->siteMap->getMenuByPageType($this->_pageId);		
			$this->_pageId		= $this->siteMapMenu->id;		
		}
		else
		{
			$this->siteMapMenu	= &$this->siteMap->getNodeById($this->_pageId);
		}

		if (!is_object($this->siteMapMenu))
		{
			$error404Page = __Config::get( 'ERROR_404');
			if ( !empty( $error404Page ) )
			{
				org_glizy_helpers_Navigation::gotoUrl( __Link::makeUrl( 'link', array( 'pageId' => $error404Page ) ) );
			}
			$e = new org_glizy_Exception(__T(GLZ_ERR_404), GLZ_E_404);
		}
	
		if (!empty($this->siteMapMenu->select)) $this->siteMapMenu = &$this->siteMap->getNodeById($this->siteMapMenu->select);
	}
	
	/* */
	function _startProcess($readPageId=true)
	{	
		$this->log( "startProcess", GLZ_LOG_SYSTEM );
		if ( $this->_logObj ) 
		{
			$this->log( array( 'Request' => __Request::getAllAsArray() ), GLZ_LOG_DEBUG );
		}
		
		if ($readPageId) $this->_readPageId();
		org_glizy_ObjectValues::set('org.glizy.application', 'pageId', $this->_pageId);
		$this->_pageType = $this->siteMapMenu->pageType;
		$pageTypeObj = &org_glizy_ObjectFactory::createPage( $this, $this->_pageType, null, array( 'pathTemplate' => org_glizy_Paths::get('APPLICATION_TEMPLATE') ) );
	
		if (!is_null($this->_rootComponent))
		{
			if (!$this->_ajaxMode)
			{
				// serve per resettare lo stato del sessionEx ad ogni caricamento delle pagine
				// altrimenti gli stati vecchi non vengono cancellati
				// quando c' un cambio di pagina e SessionEx non  usato
				org_glizy_ObjectFactory::createObject('org.glizy.SessionEx', '');
				
				$this->_rootComponent->resetDoLater();
				$this->_rootComponent->init();
				$this->_rootComponent->execDoLater();

				$this->log( "Process components", GLZ_LOG_SYSTEM );
				$this->_rootComponent->resetDoLater();
				if (method_exists($this, 'process_onStart')) $this->process_onStart();
				$this->_rootComponent->process();
				if (method_exists($this, 'process_onEnd')) $this->process_onEnd();
				$this->_rootComponent->execDoLater();

				// check if enable the PDF output
				if ( $this->getCurrentMenu()->printPdf )
				{
					$pdfPage = org_glizy_Paths::getRealPath('APPLICATION_TEMPLATE', 'pdf.php' );
					if ( $pdfPage !== false )
					{
						if ( __Request::get( 'printPdf', '0' ) )
						{
							org_glizy_ObjectValues::set( 'org.glizy.application', 'pdfMode', __Request::get( 'printPdf', '0' ) == '1' );
						}
					}
					else
					{
						$this->getCurrentMenu()->printPdf = false;
					}
				}

				$this->_rootComponent->resetDoLater();
				if (method_exists($this, 'render_onStart')) $this->render_onStart();
		
				$this->addJScoreLibraries();
	
				$output = $this->_rootComponent->render();

				if (method_exists($this, 'render_onEnd')) $this->render_onEnd();
				$this->_rootComponent->execDoLater();
				
				$headerErrorCode = __Request::get( 'glizyHeaderCode', '' );
				if ( $headerErrorCode == '404' )
				{
					header( "HTTP/1.1 404 Not Found" );
					header( "Status: 404 Not Found" );
				}
				header("Content-Type: text/html; charset=".__Config::get('CHARSET'));
				echo $output;
			}
			else
			{
				header('Cache-Control: no-cache');
				header('Pragma: no-cache');
				header('Expires: -1');

				$this->_rootComponent->resetDoLater();
				$this->_rootComponent->init();
				$this->_rootComponent->execDoLater();
	
				$targetComponent = &$this->_rootComponent->getComponentById(org_glizy_Request::get('ajaxTarget'));
				$ajaxMethod = org_glizy_Request::get('ajaxMethod', 'process_ajax');

				if (is_null($targetComponent))
				{
					// prima prova a creare i figli in modo ritardato
					// questo  usato nella gestione degli stati
					$this->_rootComponent->deferredChildCreation();
					$targetComponent = &$this->_rootComponent->getComponentById(org_glizy_Request::get('ajaxTarget'));
				
					// se il targetComponent  ancora nullo
					// prova a lanciare il process di tutti i figli
					if (is_null($targetComponent))
					{				
						$this->_rootComponent->process();
						$targetComponent = &$this->_rootComponent->getComponentById(org_glizy_Request::get('ajaxTarget'));
						if (is_null($targetComponent))
						{
							return false;
						}
					}
				}

				if ($targetComponent->getId()!=org_glizy_Request::get('ajaxTarget') || !method_exists($targetComponent, $ajaxMethod))
				{
					return false;
				}

				org_glizy_Request::remove('pageId');
				org_glizy_Request::remove('ajaxTarget');
				$result = $targetComponent->{$ajaxMethod}();
				if (!is_array($result)) $result = array('status'=> ($result===true ? 'success' : 'error'));
				if ( isset( $result['html'] ) )
				{
					header("Content-Type: text/html; charset=".__Config::get('CHARSET'));
					echo $result['html'];
				}
				else
				{
					header("Content-Type: text/html; charset=utf-8");
					if ( !function_exists( 'json_encode' ) )
					{
						glz_importLib('JSON/JSON.php');
						$json = new Services_JSON();
						echo $json->encode($result);
					}
					else
					{
						echo json_encode( $result );
					}
				}
			}
		}
		else
		{
			// TODO
			// visualizzare errore	
		}
	}	
	
	function addChild(&$component)
	{
		$this->_rootComponent = &$component;
	}
	
	function &getRootComponent()
	{
		return $this->_rootComponent;
	}


	function getOutputMode()
	{
		return $this->outputMode;
	}

	function getPageId()
	{
		return strtolower( $this->_pageId );
	}


	function setPageId($id)
	{
		$this->_pageId = $id;
		$this->siteMapMenu = &$this->siteMap->getNodeById($this->_pageId);
	}

	function getPageType()
	{
		return $this->_pageType;
	}

	function &getCurrentMenu()
	{
		return $this->siteMapMenu;
	}

	function &getCurrentUser()
	{
		$user = &org_glizy_ObjectValues::get('org.glizy', 'user');
		return $user;
	}
	
	function &getSiteMap()
	{
		return $this->siteMap;
	}

	function getLanguageId()
	{
		return $this->_languageId;
	}

	function setLanguageId($value)
	{
		$this->_languageId = $value;
	}
	
	function getLanguage()
	{
		return strtolower($this->_language);
	}

	function setLanguage($value)
	{
		$value = strtolower($value);
		if ($this->_language != $value)
		{
			$this->_language = $value;
			$this->_loadLocale();
		}
	}	

	function _loadLocale()
	{
		// importa i file di localizzazione
		if (file_exists(org_glizy_Paths::getRealPath('CORE_CLASSES').'org/glizy/locale/'.$this->getLanguage().'.php'))
		{
			require_once(org_glizy_Paths::getRealPath('CORE_CLASSES').'org/glizy/locale/'.$this->getLanguage().'.php');
		}
		else
		{
			require_once(org_glizy_Paths::getRealPath('CORE_CLASSES').'org/glizy/locale/en.php');
		}
	}
	
	function addJScoreLibraries()
	{
		if (!org_glizy_ObjectValues::get('org.glizy.JS.Core', 'add', false) && __Config::get( 'GLIZY_ADD_CORE_JS' ) )
		{
			org_glizy_ObjectValues::set('org.glizy.JS.Core', 'add', true);
			$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('Glizy.js?v='.GLZ_CORE_VERSION), 'head');
			$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile($this->getLanguage().'.js', 'locale/'), 'head');
		}
	}
	
	function addJSLibCore()
	{
		if (!org_glizy_ObjectValues::get('org.glizy.JS.lib', 'add', false) && __Config::get( 'GLIZY_ADD_JS_LIB' ) )
	{
			org_glizy_ObjectValues::set('org.glizy.JS.lib', 'add', true);
			if ( __Config::get( 'GLIZY_JS_LIB' ) == 'jquery' )
		{
			// jQuery
				$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::linkStaticJSfile( 'jquery/' . __Config::get('GLIZY_JQUERY' ) ), 'head');
				if ( __Config::get( 'GLIZY_ADD_JQUERYUI_JS' ) )
				{
					$this->_rootComponent->addOutputCode( org_glizy_helpers_JS::linkStaticJSfile( 'jquery/jquery-ui/' . __Config::get('GLIZY_JQUERYUI' ) ), 'head');
					$this->_rootComponent->addOutputCode( org_glizy_helpers_CSS::linkStaticCSSfile( 'jquery/jquery-ui/' . __Config::get('GLIZY_JQUERYUI_THEME' ) ), 'head');
				}
			}
			else
			{
				// mootools
				$this->_rootComponent->addOutputCode( org_glizy_helpers_JS::linkCoreJSfile2( 'mootools/' . __Config::get('GLIZY_MOOTOOLS' ) ), 'head' );
				$this->_rootComponent->addOutputCode( org_glizy_helpers_JS::linkCoreJSfile2( 'mootools/' . __Config::get('GLIZY_MOOTOOLS_MORE' ) ), 'head' );
			}
		}
	}
	
	function addLightboxJsCode()
	{
		if (!org_glizy_ObjectValues::get('org.glizy.JS.Lightbox', 'add', false) && __Config::get( 'GLIZY_ADD_JS_LIB' ) )
		{
			org_glizy_ObjectValues::set('org.glizy.JS.Lightbox', 'add', true);
			$this->addJSLibCore();
		
			if ( __Config::get( 'GLIZY_JS_LIB' ) == 'jquery' )
			{
				// TODO
			}
			else if ( __Config::get( 'GLIZY_JS_LIB' ) == 'mootools' )
			{
				$this->_rootComponent->addOutputCode( org_glizy_helpers_CSS::linkCoreCSSfile2('milkbox/milkbox.css' ), 'head' );
				$this->_rootComponent->addOutputCode( org_glizy_helpers_JS::linkCoreJSfile2('milkbox/milkbox.js' ), 'head' );
				$this->_rootComponent->addOutputCode( org_glizy_helpers_JS::JScode( 'window.addEvent(\'domready\', function() { Milkbox.setOptions({autoPlay: true, autoPlayDelay: Glizy && Glizy.slideShowSpeed ? Glizy.slideShowSpeed : 10})});' ), 'head' );	
			}
		}
	}
	
	function addZoomJsCode()
	{
		if (!org_glizy_ObjectValues::get('org.glizy.JS.Zoom', 'add', false) && __Config::get( 'GLIZY_ADD_JS_LIB' ) )
		{
			org_glizy_ObjectValues::set('org.glizy.JS.Zoom', 'add', true);
			$this->addJSLibCore();
			$this->_rootComponent->addOutputCode( org_glizy_helpers_JS::linkStaticJSfile('OpenSeadragon/OpenSeadragon.js' ), 'head' );
			$this->_rootComponent->addOutputCode( '<div id="zoomContainer"></div>' );
		}
	}
	
	function addValidateJsCode( $formName=null )
	{
		if (!org_glizy_ObjectValues::get('org.glizy.JS.Validate', 'add', false) && __Config::get( 'GLIZY_ADD_JS_LIB' ) )
		{
			// Validate
			org_glizy_ObjectValues::set('org.glizy.JS.Validate', 'add', true);
		 	$this->addJSLibCore();
			if ( __Config::get( 'GLIZY_JS_LIB' ) == 'jquery' )
			{
				if ( file_exists( org_glizy_Paths::get('STATIC_DIR').'jquery/jquery.validationEngine/jquery.validationEngine-'.$this->getLanguage().'.js' ) )
				{
					$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::linkStaticJSfile( 'jquery/jquery.validationEngine/jquery.validationEngine-'.$this->getLanguage().'.js' ), 'head');
				}
				$this->_rootComponent->addOutputCode( org_glizy_helpers_JS::linkStaticJSfile( 'jquery/jquery.validationEngine/jquery.validationEngine.js' ), 'head');
				$this->_rootComponent->addOutputCode( org_glizy_helpers_CSS::linkStaticCSSfile( 'jquery/jquery.validationEngine/validationEngine.jquery.css' ), 'head');
			} 
			else if ( __Config::get( 'GLIZY_JS_LIB' ) == 'mootools' )
			{
			if ( file_exists( org_glizy_Paths::get('CORE_STATIC_DIR').'formcheck/lang/'.$this->getLanguage().'.js' ) )
			{
				$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile2('formcheck/lang/'.$this->getLanguage().'.js'), 'head');
			}
			
			$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile2('formcheck/formcheck.js'), 'head');
			$this->_rootComponent->addOutputCode(org_glizy_helpers_CSS::linkCoreCSSfile2('formcheck/theme/classic/formcheck.css'), 'head');
		}
		}
		
		if ( !is_null( $formName ) && __Config::get( 'GLIZY_ADD_JS_LIB' ) )
		{
			if ( __Config::get( 'GLIZY_JS_LIB' ) == 'jquery' )
			{
				$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::JScode( '$(document).ready(function() { $("#'.$formName.'").validationEngine( "attach", { validationEventTrigger: "none", scroll: false, showAllErrors: false } ); });' ), 'head');
				
//				$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::JScode( '$(document).ready(function() { $("#'.$formName.'").validationEngine(); });' ), 'head');
			} 
			else if ( __Config::get( 'GLIZY_JS_LIB' ) == 'mootools' )
		{
			$this->_rootComponent->addOutputCode(org_glizy_helpers_JS::JScode( 'window.addEvent(\'domready\', function(){ var '.$formName.'Validation = new FormCheck(\''.$formName.'\'); });' ), 'head');
		}
	}
	}

	/* events listener */
	function login()
	{
		$this->log( "login", GLZ_LOG_SYSTEM );
		if (org_glizy_Session::get('glizy.userLogged'))
		{
			$this->log( "user is logged", GLZ_LOG_SYSTEM );
			$user = org_glizy_Session::get('glizy.user');

			// crea l'utente
			$this->_user = &org_glizy_ObjectFactory::createObject('org.glizy.application.User', $user);
			org_glizy_ObjectValues::setByReference('org.glizy', 'user', $this->_user);
			
			if (org_glizy_Config::get('USER_LOG'))
			{
				$this->log( "log user access", GLZ_LOG_SYSTEM );
				$arLog = &org_glizy_ObjectFactory::createModel('org.glizy.models.UserLog');
				$arLog->load($user['logId']);
				$arLog->userlog_FK_user_id = $user['id'];
				$arLog->save();
			}
		}
		else
		{
			$this->log( "user not logged", GLZ_LOG_SYSTEM );
			// utente finto
			$user = 0;
			// crea l'utente
			$this->_user = &org_glizy_ObjectFactory::createObject('org.glizy.application.User', $user);
			org_glizy_ObjectValues::setByReference('org.glizy', 'user', $this->_user);		
		}
	}
	
	function isAdmin()
	{
		return false;
	}
	
	function canViewPage()
	{
		return true;
	}
}
?>