<?php
/* SVN FILE: $Id$ */

/**
 * Application  class.
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.application
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */

glz_import('org.glizy.application.Application');

class org_glizy_application_ApplicationDB extends org_glizy_application_Application
{
	var $_aclPage;
	var $_siteProperty;
	var $_templateName;
	
	function _initLanguage()
	{
		$this->log( "initLanguage", GLZ_LOG_SYSTEM );
		// inizializza la lingua
		$this->_language = org_glizy_Session::get('glizy.language', NULL);
		$this->_languageId = org_glizy_Session::get('glizy.languageId', NULL);
		if (is_null($this->_languageId))
		{
			$this->log( "Read defaul language", GLZ_LOG_SYSTEM );
			$ar = org_glizy_ObjectFactory::createModel('org.glizy.models.Language');
			$ar->language_isDefault = 1;
			$ar->language_FK_site_id = __Config::get( 'SITE_ID' );
			$ar->find();
			$this->_language = $ar->language_code;
			$this->_languageId = $ar->language_id;
			org_glizy_Session::set('glizy.language', $this->_language);
			org_glizy_Session::set('glizy.languageId', $this->_languageId);
		}

		// importa i file di localizzazione
		if (file_exists(org_glizy_Paths::getRealPath('CORE_CLASSES').'org/glizy/locale/'.$this->getLanguage().'.php'))
		{
			$this->log( "Import locale file", GLZ_LOG_SYSTEM );
			require_once(org_glizy_Paths::getRealPath('CORE_CLASSES').'org/glizy/locale/'.$this->getLanguage().'.php');
		}
	}

	function _initSiteMap()
	{
		$this->log( "initSiteMap", GLZ_LOG_SYSTEM );
		$this->siteMap = &org_glizy_ObjectFactory::createObject('org.glizy.application.SiteMapDB');
		$this->siteMap->getSiteArray();	
		
		// controlla se l'utente ha i permessi per modificare la pagina
		// per velocizzare vengono precaricate tutte le relazioni in memoria
		$this->_aclPage = array();
		if ( __Config::get( 'ACL_ENABLED' ) )
		{
			$this->_aclPage = __Session::get( 'glizy.aclFront', NULL );
			if ( is_null( $this->_aclPage ) )
			{
				$this->_aclPage = array();
			$it = org_glizy_ObjectFactory::createModelIterator( 'org.glizy.models.Join', 'all', array( 'filters' => array( 'join_objectName' => 'menus_tbl#rel_aclFront' ) ) );
			while( $it->hasMore() )
			{
				$arC = $it->current();
				$it->next();
				if ( !isset( $this->_aclPage[ $arC->join_FK_source_id ] ) )
				{
					$this->_aclPage[ $arC->join_FK_source_id ] = array();
				}
				$this->_aclPage[ $arC->join_FK_source_id ][] = $arC->join_FK_dest_id ;
			}
				
				// scorre tutti i men per attribuire l'acl ai men che non ce l'hanno
				// ereditandola dal padre
				$siteMap = &org_glizy_ObjectFactory::createObject('org.glizy.application.SiteMapDB');
				$siteMap->getSiteArray();
				$siteMapIterator = &org_glizy_ObjectFactory::createObject('org.glizy.application.SiteMapIterator', $siteMap);
				while (!$siteMapIterator->EOF)
				{
					$n = $siteMapIterator->getNode();
					$siteMapIterator->moveNext();
					
					if ( !isset( $this->_aclPage[ $n->id ] ) )
					{
						$n2 = $n;
						while ( true )
						{
							if ( $n2->parentId == 0 ) break;
							$parentNode =  $n2->parentNode();
							$n2 = $parentNode;
							if ( isset( $this->_aclPage[ $parentNode->id ] ) )
							{
								$this->_aclPage[ $n->id ] = $this->_aclPage[ $parentNode->id ];
								break;
							}
						}
					}
				}
				
				__Session::set( 'glizy.aclFront', $this->_acl );
			}
		}
	}

	/* */
	function _startProcess()
	{	
		$this->log( "startProcess", GLZ_LOG_SYSTEM );
		$language = org_glizy_Request::get('language', NULL);
		
		if (!is_null($language) && $language!=$this->_language)
		{
			// cambio lingua
			$this->log( "change language", GLZ_LOG_SYSTEM );
			$ar = org_glizy_ObjectFactory::createModel('org.glizy.models.Language');
			$ar->language_code = $language;
			$ar->language_isDefault = NULL;
			$ar->find();
			$this->_language = $ar->language_code;
			$this->_languageId = $ar->language_id;
			org_glizy_Session::set('glizy.language', $this->_language);
			org_glizy_Session::set('glizy.languageId', $this->_languageId);
			org_glizy_ObjectValues::setByReference('org.glizy', 'application', $this);

			$this->_initSiteMap();
			
			// importa i file di localizzazione
			if (file_exists(org_glizy_Paths::getRealPath('CORE_CLASSES').'org/glizy/locale/'.$this->getLanguage().'.php'))
			{
				$this->log( "import locale file", GLZ_LOG_SYSTEM );
				require_once(org_glizy_Paths::getRealPath('CORE_CLASSES').'org/glizy/locale/'.$this->getLanguage().'.php');
			}
		}
		
		$this->readSiteProperties();
		$this->setTemplateFolder();
		$this->_readPageId();
		
		if ($this->siteMapMenu->isVisible===false)
		{
			while(true)
			{
				$parentMenu = &$this->siteMapMenu->parentNode();
				
				if (is_null($parentMenu))
				{
					// ERROR
					$e = new org_glizy_Exception(array('[%s] %s', $this->getClassName(), __T(GLZ_ERR_EMPTY_APP_PATH)));
				}
				
				$this->siteMapMenu = &$parentMenu;
				if ($parentMenu->isVisible===true)
				{
					$this->_pageId = $this->siteMapMenu->id;
					break;
				}
			}
		}
		
		// TODO da risolvare in modo migliore
		// creando un nuovo menu_type
		if ($this->siteMapMenu->pageType=='Empty')
		{
			$currentPage = &$this->siteMapMenu;
			while (true)
			{
				$childNodes = $currentPage->childNodes();
				if (!count($childNodes)) 
				{
					org_glizy_helpers_Navigation::gotoUrl(GLZ_HOST);
					break;
				}
				$tempPage = &$childNodes[0];
				$currentPage = &$tempPage;
				if ($currentPage->pageType!='Empty')
				{
					$this->siteMapMenu = &$currentPage;
					$this->_pageId = $currentPage->id;
					break;
				}
			}
		}
		
		parent::_startProcess(false);
	}
	
	function canViewPage( $page=null )
	{
		if ( is_null( $page ) )
		{
			$page = $this->_pageId;
		}
		
		$user = &$this->getCurrentUser();
		if ( isset( $this->_aclPage[ $page ] ) && $user->groupId != 1 )
		{
			return in_array( $user->groupId, $this->_aclPage[ $page ] );
		}
		else return true;
	}
	
	function getSiteProperty()
	{
		return $this->_siteProperty;
	}
	
	function getTemplateName()
	{
		return $this->_templateName;
	}
	
	private function readSiteProperties()
	{
		$siteProp = unserialize(org_glizy_Registry::get(__Config::get( 'REGISTRY_SITE_PROP' ).$this->getLanguageId(), ''));
		if (!is_array($siteProp))
		{
			// if the site properties are not defined
			// try to read the properties from default language			
			$ar = org_glizy_ObjectFactory::createModel('org.glizy.models.Language');
			$ar->language_isDefault = 1;
			$ar->find();
			$siteProp = org_glizy_Registry::get(__Config::get( 'REGISTRY_SITE_PROP' ).$ar->language_id, '');
			org_glizy_Registry::set(__Config::get( 'REGISTRY_SITE_PROP' ).$this->getLanguageId(), $siteProp);
			$siteProp = unserialize($siteProp);
		}
		if (!is_array($siteProp))
		{
			$siteProp = array();
		}
		$this->_siteProperty = $siteProp;
	}
	
	private function setTemplateFolder()
	{
		// read the template name
		$this->_templateName = org_glizy_Registry::get( __Config::get( 'REGISTRY_TEMPLATE_NAME' ), '');
		if (org_glizy_Request::get('template', '') != '')
		{
			$this->_templateName = org_glizy_Request::get('template', '');
		}

		$browser = strpos($_SERVER['HTTP_USER_AGENT'],"iPhone") || strpos($_SERVER['HTTP_USER_AGENT'],"Android");
		if ($browser == true)
		{ 
			if ( file_exists( org_glizy_Paths::get('APPLICATION_STATIC').'templates/'.$this->_templateName.'-mobile' ) )
			{
				$this->_templateName .= '-iPhone';
			}
			else if ( file_exists( org_glizy_Paths::get('APPLICATION_STATIC').'templates/mobile' ) )
			{
				$this->_templateName = 'iPhone';
			}
		}
		if (empty($this->_templateName))
		{
			// TODO
			// tradurre il messaggio di errore
			$this->triggerError('Please specify the template');
		}
		org_glizy_Paths::set('APPLICATION_TEMPLATE', org_glizy_Paths::get('APPLICATION_STATIC').'templates/'.$this->_templateName.'/');
		org_glizy_Paths::set('APPLICATION_TEMPLATE_DEFAULT', org_glizy_Paths::get('APPLICATION_STATIC').'templates/Default/');
	}
}
?>