<?php
/**
 * 
 * @copyright Copyright(c) 2005 Daniele Ugoletti. All rights reserved.
 * @license http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 *
 * GLIZY is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See COPYRIGHT.php for copyright notices and details.
 *
 * @author		Daniele Ugoletti <daniele@ugoletti.com>
 * @version		
 * @package		Glizy core
 * @category	
 */

glz_import('org.glizy.compilers.Compiler');

class org_glizy_compilers_PageType extends org_glizy_compilers_Compiler
{	
	var $_classSource		= '';
	var $_customClassSource	= '';
	var $_importedPaths = array();
	var $_className			= '';
	var $_path 				= false;
	
	function compile($options)
	{
		$this->initOutput();
		
		if ( isset( $options[ 'pathTemplate' ] ) )
		{
			$fileName = $options[ 'pathTemplate' ].'/pageTypes/'.$options[ 'pageType' ];
			if ( file_exists( $fileName ) )
			{
				$this->_fileName = $fileName;
			}
		}
		
		$pageXml = org_glizy_ObjectFactory::createObject( 'org.glizy.parser.XML' );
		$pageXml->loadAndParseNS( $this->_fileName );
		$pageRootNode 			= $pageXml->documentElement;
		$registredNameSpaces 	= $pageXml->namespaces;
		$registredNameSpaces['glz'] = 'org.glizy.components';
		$idPrefix 				= isset($options['idPrefix']) ? $options['idPrefix'] : '';
		$this->_path			= $options['path'];

		// include i componenti usati
		foreach ($registredNameSpaces as $key=>$value)
		{
			if ($key!='glz' && substr($value, -1, 1)=='*' && !in_array($value, $this->_importedPaths)) 
			{
				glz_import($value);
				$this->output .= 'glz_import(\''.$value.'\')'.GLZ_COMPILER_NEWLINE;
				$this->_importedPaths[] = $value;
			}
		}
	
	
		$this->_className = glz_basename($this->_cacheObj->getFileName());
		$this->_classSource .= 'class '.$this->_className.'{'.GLZ_COMPILER_NEWLINE2;
		$this->_classSource .= 'function '.$this->_className.'(&$application, $skipImport=false) {'.GLZ_COMPILER_NEWLINE2;
		$counter = 0;
		$this->_compileXml($pageRootNode, $registredNameSpaces, $counter, '$application', '\''.$idPrefix.'\'.');
		$this->_classSource .= '}'.GLZ_COMPILER_NEWLINE2;
		$this->_classSource .= '}'.GLZ_COMPILER_NEWLINE2;

		$this->output .= $this->_classSource;
		$this->output .= $this->_customClassSource;
		
		return $this->save();
	}
	
	
	function _compileXml(&$node, &$registredNameSpaces, &$counter, $parent='NULL', $idPrefix='')
	{
		if ($node->nodeType == XML_COMMENT_NODE) return;
		
		// sostituisce i caratteri speciali all'interno del nome del tag
		// per poter verificare se  stato deifnito un metodo aaposito
		// per compilare il tag
		$methodName = 'compile_'.preg_replace('/[\-\#\:]/', '', $node->nodeName);

		if (method_exists($this, $methodName))
		{
			$this->$methodName($node, $registredNameSpaces, $counter, $parent, $idPrefix);
		}
		else
		{
			$this->_classSource .= '// TAG: '.$node->nodeName.GLZ_COMPILER_NEWLINE2;
			
			$componentClassInfo = $this->_getComponentClassInfo($node->nodeName, $registredNameSpaces);
			$componentId = $node->hasAttribute( 'id' ) ? $node->getAttribute( 'id' ) : 'c'.md5(uniqid(rand(), true));
			$componentIdPrefix = $idPrefix;
			
			if (!empty($componentClassInfo['classPath']) && !in_array($componentClassInfo['classPath'], $this->_importedPaths))
			{
				glz_import($componentClassInfo['classPath']);
				$this->output .= 'glz_import(\''.$componentClassInfo['classPath'].'\')'.GLZ_COMPILER_NEWLINE;
				$this->_importedPaths[] = $componentClassInfo['classPath'];
			}
			if (class_exists($componentClassInfo['className']))
			{
				$n = NULL;
				$c = new $componentClassInfo['className']($n, $n);
				if (method_exists($c, 'compile'))
				{
					$c->compile($this, $node, $registredNameSpaces, $counter, $parent, $idPrefix, $componentClassInfo, $componentId );
					return true;
					// TODO: DA VERIFICARE
					//  giusto che esca dalla funzione?
					// cosa succede se il componente che ha un custom compile
					// ha dei tag figli?
				}
				if (method_exists($c, 'compileAddPrefix'))
				{
					$componentIdPrefix = $c->compileAddPrefix($this, $node, $componentId, $idPrefix);
				}
			}
			
			$this->_classSource .= '$n'.$counter.' = &org_glizy_ObjectFactory::createComponent(\''.$componentClassInfo['classPath'].'\', $application, '.$parent.', \''.$node->nodeName.'\', '.$idPrefix.'\''.$componentId.'\', \''.$componentId.'\', $skipImport)'.GLZ_COMPILER_NEWLINE;
			
			if ($parent!='NULL')
			{
				$this->_classSource .= $parent.'->addChild($n'.$counter.')'.GLZ_COMPILER_NEWLINE;
			}
			
			if (count($node->attributes))
			{
				// compila  gli attributi
				//foreach ($node->attributes as $key=>$value)
				///{
				//	if ($key!='id') $this->_classSource .= '$n'.$counter.'->setAttribute(\''.$key.'\', \''.addslashes($value).'\')'.GLZ_COMPILER_NEWLINE;
				//}
				// compila  gli attributi
				$this->_classSource .= '$attributes = array(';
				foreach ( $node->attributes as $index=>$attr )
				{
					if ($attr->name!='id') 
					{
						// NOTA: su alcune versioni di PHP (es 5.1)  empty( $attr->prefix ) non viene valutato in modo corretto
						$prefix = $attr->prefix == "" ||  is_null( $attr->prefix ) ? "" : $attr->prefix.":";
						$this->_classSource .= '\''.$prefix.$attr->name.'\' => \''.addslashes($attr->value).'\', ';
					}
				}
		
				$this->_classSource .= ')'.GLZ_COMPILER_NEWLINE;
				$this->_classSource .= '$n'.$counter.'->setAttributes( $attributes )'.GLZ_COMPILER_NEWLINE;
			}
			
			$idPrefix = $componentIdPrefix;
			$oldcounter = $counter;
			
			foreach( $node->childNodes as $nc )
			{
				$counter++;
				$this->_compileXml($nc, $registredNameSpaces, $counter, '$n'.$oldcounter, $idPrefix);
			}
		}
	}
	
	function _getComponentClassInfo($componentName, &$registredNameSpaces)
	{
		$result = array('className' => '', 'classPath' => '');
		$componentClassName = explode(':', $componentName);
		if (count($componentClassName)==2)
		{
			$nameSpace 			= $componentClassName[0];
			$componentClassName = $componentClassName[1];
			if (array_key_exists($nameSpace, $registredNameSpaces))
			{
				if ($registredNameSpaces[$nameSpace]!='*')
				{
					$componentClassName = rtrim($registredNameSpaces[$nameSpace], '.*').'.'.$componentClassName;
				}
				$result['classPath'] = $componentClassName;
				$componentClassName = str_replace('*', '', $componentClassName);
				$componentClassName = str_replace('.', '_', $componentClassName);				
				$result['className'] = $componentClassName;
			}
			else
			{
				// TODO
				// namespace non definito
				// visualizzare un errore
				
			}
		}
		else
		{
			$result['className'] = $componentName;
		}
		return $result;
	}
	
	function compile_cdatasection(&$node, &$registredNameSpaces, &$counter, $parent='NULL', $idPrefix='')
	{
		$this->compile_text($node, $registredNameSpaces, $counter, $parent);
	}

	function compile_text(&$node, &$registredNameSpaces, &$counter, $parent='NULL', $idPrefix='')
	{
		$this->_classSource .= '$tagContent = <<<EOD'.GLZ_COMPILER_NEWLINE2;
		$this->_classSource .=  str_replace('$', '\$', $node->nodeValue).GLZ_COMPILER_NEWLINE2;
		$this->_classSource .= 'EOD'.GLZ_COMPILER_NEWLINE;
		$this->_classSource .= $parent.'->setContent($tagContent)'.GLZ_COMPILER_NEWLINE;		
	}
	
	function compile_glzinclude(&$node, &$registredNameSpaces, &$counter, $parent='NULL', $idPrefix='')
	{
		$src = $node->getAttribute( 'src' );
		if ( strpos( $src, '.xml' ) === false ) 
		{
			$src .= '.xml';
		}
		if ( file_exists( dirname( $this->_fileName ).'/'.$src ) )
		{
			$src = dirname( $this->_fileName ).'/'.$src;
		}
		else
		{
			$src = glz_findClassPath( $node->getAttribute( 'src' ) );
		}
		if ( is_null( $src ) )
		{
			die( '[ERROR] glz:include file not found: '.$node->getAttribute( 'src' ) );
		}
		$this->_classSource .= '// include: '.$src.GLZ_COMPILER_NEWLINE2;
		
		$srcXml = file_get_contents( $src );
		if ( isset($node->attributes['template'] ) )
		{
			$template = explode( ';', $node->attributes['template'] );
			foreach( $template as $v )
			{
				list( $name, $value ) = explode( ':', $v );
				$srcXml = str_replace( '##'.$name.'##', $value, $srcXml );
			}
		}
		
		$includeXML = org_glizy_ObjectFactory::createObject( 'org.glizy.parser.XML' );
		$includeXML->loadXmlAndParseNS( $srcXml );
		$registredNameSpaces = array_merge( $registredNameSpaces, $includeXML->namespaces );
		// include i componenti usati
		foreach ($registredNameSpaces as $key=>$value)
		{
			if ($key!='glz' && substr($value, -1, 1)=='*' && !in_array($value, $this->_importedPaths)) 
			{
				glz_import($value);
				$this->output .= 'glz_import(\''.$value.'\')'.GLZ_COMPILER_NEWLINE;
				$this->_importedPaths[] = $value;
			}
			if ( !isset( $registredNameSpaces[ $key ] ) )
			{
				$registredNameSpaces[ $key ] = $value;
			}
		}
	
		$this->_compileXml($includeXML->documentElement, $registredNameSpaces, $counter, $parent, $idPrefix);
	}
	
	function getPath()
	{
		return $this->_path;
	}
}
?>