<?php
/* SVN FILE: $Id$ */

/**
 * 
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.components
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */
 
glz_import('org.glizy.components.Component');
glz_import('org.glizy.helpers.CSS');
glz_import('org.glizy.helpers.JS');

class org_glizy_components_Date extends org_glizy_components_Component
{
	/**
	 * Return the component information
	 * 
	 * @return	array	component informaton.
	 * @access	public
	 * @static
	 */
	function getInfo()
	{
		$info 					= parent::info();
		$info['name']			= 'Date';
		$info['class'] 			= 'org.glizy.components.Date';
		$info['package'] 		= 'Glizy';
		$info['version'] 		= GLZ_CORE_VERSION;
		$info['author'] 		= 'Daniele Ugoletti';
		$info['author-email'] 	= 'daniele@ugoletti.com';
		$info['url'] 			= 'http://www.ugoletti.com';
		return $info;
	}
	
	/**
	 * Init
	 * 
	 * @return	void
	 * @access	public
	 */
	function init()
	{
		// define the custom attributes
		$this->defineAttribute('label',				false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('adm:required',		false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('model:required',	false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('adm:requiredMessage',	false, 	NULL,	COMPONENT_TYPE_STRING);		
		$this->defineAttribute('adm:showTime',	false, 	false,	COMPONENT_TYPE_BOOLEAN);		
		
		// call the superclass for validate the attributes
		parent::init();
	}
	
	/**
	 * Process
	 * 
	 * @return	boolean	false if the process is aborted
	 * @access	public
	 */
	function process()
	{
		$this->_content = $this->_parent->loadContent($this->getId());
		if ($this->_content=='0000-00-00') $this->_content = '';
	}
	
	function render_html()
	{
		$output  = '<input type="text" size="10" id="'.$this->getId().'" name="'.$this->getId().'" value="'.$this->_content.'" />';
		$output  .= '<input type="reset" onclick="return showCalendar(\''.$this->getId().'\', \'%d/%m/%Y\');" value=" ... ">';

		$output = org_glizy_helpers_Html::label($this->getAttributeString('label'), $this->getId(), '', $output, array( 'class' => '' ), false);

		$this->addOutputCode($output);

		if (!org_glizy_ObjectValues::get('org.glizy.JS.DateScript', 'add', false))
		{
			org_glizy_ObjectValues::set('org.glizy.JS.DateScript', 'add', true);
			$rootComponent = &$this->getRootComponent();
			$rootComponent->addOutputCode(org_glizy_helpers_CSS::linkCoreCSSfile('calendar-system.css', '../js/jscalendar/'), 'head');
			$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('calendar.js', 'jscalendar/'), 'head');
			$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('calendarFunc.js', 'jscalendar/'), 'head');
			$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('calendar-'.$this->_application->getLanguage().'.js', 'jscalendar/lang/'), 'head');
			$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('calendar-setup.js', 'jscalendar/'), 'head');
		}
	}
	
	function render_jsediting()
	{
		$output = array();
		$output['id'] 		= $this->getId();
		$output['type'] 	= $this->getAttribute( 'adm:showTime' ) ? 'datetime' :  'date';
		$output['label'] 	= $this->getAttribute('label');
		$output['value'] 	=  $this->_content;
		
		if ($this->getAttribute('adm:required') || $this->getAttribute('model:required'))
		{
			$output['required'] = 'Glizy.validateForm';
			if (!is_null($this->getAttribute('adm:requiredMessage')))
			{
				$output['requiredMessage'] = $this->getAttribute('adm:requiredMessage');
			}
		}
		$this->addOutputCode($output);
		
		if (!org_glizy_ObjectValues::get('org.glizy.JS.DateScript', 'add', false))
		{
			org_glizy_ObjectValues::set('org.glizy.JS.DateScript', 'add', true);
			$rootComponent = &$this->getRootComponent();
			$rootComponent->addOutputCode(org_glizy_helpers_CSS::linkCoreCSSfile('calendar-system.css', '../js/jscalendar/'), 'head');
			$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('calendar.js', 'jscalendar/'), 'head');
			$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('calendar-'.$this->_application->getLanguage().'.js', 'jscalendar/lang/'), 'head');
			$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('calendar-setup.js', 'jscalendar/'), 'head');
		}
	}
}
?>