<?php
/* SVN FILE: $Id$ */

/**
 * 
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.components
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */
 
glz_import('org.glizy.components.HtmlFormElement');

class org_glizy_components_Input extends org_glizy_components_HtmlFormElement
{
	/**
	 * Return the component information
	 * 
	 * @return	array	component informaton.
	 * @access	public
	 * @static
	 */
	function getInfo()
	{
		$info 					= parent::info();
		$info['name']			= 'Input';
		$info['class'] 			= 'org.glizy.components.Input';
		$info['package'] 		= 'Glizy';
		$info['version'] 		= GLZ_CORE_VERSION;
		$info['author'] 		= 'Daniele Ugoletti';
		$info['author-email'] 	= 'daniele@ugoletti.com';
		$info['url'] 			= 'http://www.ugoletti.com';
		return $info;
	}
	
	
	/**
	 * Init
	 * 
	 * @return	void
	 * @access	public
	 */
	function init()
	{
		// define the custom attributes
		$this->defineAttribute('defaultValue',		false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('bindTo',			false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('cols',				false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('cssClass',			false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('disabled',			false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('label',				false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('minLength',			false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('maxLength',			false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('rows',				false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('readOnly',			false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('size',				false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('value',				false, 	NULL,		COMPONENT_TYPE_STRING);
		$this->defineAttribute('wrapLabel',			false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('wrap',				false, 	'off',	COMPONENT_TYPE_STRING);
		$this->defineAttribute('type',				false, 	'text',	COMPONENT_TYPE_STRING);
		$this->defineAttribute('htmlEditor',		false, 	false,	COMPONENT_TYPE_BOOLEAN);	// TODO
		$this->defineAttribute('required',			false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('requiredMessage',	false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('validationType',	false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('validationMessage',	false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('title',				false, 	'',	COMPONENT_TYPE_STRING);
		$this->defineAttribute('placeholder',		false, 	'',	COMPONENT_TYPE_STRING);
		
		// call the superclass for validate the attributes
		parent::init();
	}


	/**
	 * Process
	 * 
	 * @return	boolean	false if the process is aborted
	 * @access	public
	 */	
	function process()
	{	
		$this->_content = $this->getAttribute('value');
		if (is_object($this->_content))
		{
			$contentSource = &$this->getAttribute('value');
			$this->_content = $contentSource->loadContent($this->getId(), $this->getAttribute('bindTo'));
		}
		else if (is_null($this->_content))
		{
			$this->_content = $this->_parent->loadContent($this->getId(), $this->getAttribute('bindTo'));
		}
		else
		{
			$this->_content = html_entity_decode( $this->_content );
		}
		if (empty($this->_content)) 
		{
			$this->_content = $this->getAttribute('defaultValue');
			
			if ( method_exists($this->_parent, 'setFilterValue') )
			{		 
				$bindTo = $this->getAttribute('bindTo');
				$this->_parent->setFilterValue(!empty($bindTo) ? $bindTo : $this->getId(), $this->_content);
			}
		}

	}

	/**
	 * Render
	 * 
	 * @return	void
	 * @access	public
	 */
	function render_html()
	{
		$attributes 				= array();
		$attributes['id'] 			= $this->getId();
		$attributes['name'] 		= $this->getOriginalId();
		$attributes['disabled'] 	= $this->getAttribute('disabled') ? 'disabled' : '';
		$attributes['readonly'] 	= $this->getAttribute('readOnly') ? 'readonly' : '';
		$attributes['title'] 		= $this->getAttributeString('title');
		$attributes['placeholder'] 		= $this->getAttributeString('placeholder');
		if ( empty( $attributes['title'] ) )
		{
			$attributes['title'] 		= $this->getAttributeString('label');
		}
		$attributes['class'] 		= $this->getAttribute('cssClass');
		$attributes['class'] 		.= (!empty($attributes['class']) ? ' ' : '').($this->getAttribute('required') ? 'required' : '');
		
		if ($this->getAttribute('type')=='multiline')
		{
			$attributes['cols'] 		= $this->getAttribute('cols');
			$attributes['rows'] 		= $this->getAttribute('rows');
			$attributes['wrap'] 		= $this->getAttribute('wrap');
			
			$output  = '<textarea '.$this->_renderAttributes($attributes).'>';
			$output .= $this->encodeOuput($this->_content);
			$output .= '</textarea>';
		}
		else
		{
			$attributes['type'] 		= $this->getAttribute('type');
			$attributes['maxLength'] 	= $this->getAttribute('maxLength');
			$attributes['size'] 		= $this->getAttribute('size');	
			$attributes['value'] 		= $this->encodeOuput($this->_content);
			
			$output  = '<input '.$this->_renderAttributes($attributes).'/>';
		}
		
		$output = org_glizy_helpers_Html::label($this->getAttributeString('label'), $this->getId(),  $this->getAttribute('wrapLabel'), $output, array('class' => ($this->getAttribute('required') ? 'required' : '')), false);
		$this->addOutputCode($output);
	}
	
	function render_jsediting()
	{
		$output = array();
		$output['id'] 		= $this->getId();
		$output['label'] 	= $this->getAttribute('label');
		$output['value'] 	= javascript_to_html(strval($this->getText()));
		$output['cssClass'] 	= $this->getAttribute('cssClass');
		
		if ($this->getAttribute('required'))
		{
			$output['required'] = 'Glizy.validateForm';
			if (!is_null($this->getAttribute('requiredMessage')))
			{
				$output['requiredMessage'] = $this->getAttribute('requiredMessage');
			}
		}
		
		if ($this->getAttribute('type')=='multiline')
		{
			$output['type'] 	= 'longtext';
			$output['htmlEditor'] 	= $this->getAttribute('htmlEditor')===true ? 1 : 0;

				$output['value']  = str_replace(array("\r\n", "\n", "\r"), "<br>", $output['value']);
			if (!$this->getAttribute('htmlEditor'))
			{
				if (!is_null($this->getAttribute('maxLength'))) $output['maxlength'] = $this->getAttribute('maxLength');
			}
			if ($this->getAttribute('cols')!='') $output['cols'] = $this->getAttribute('cols');
			if ($this->getAttribute('rows')!='') $output['rows'] = $this->getAttribute('rows');			
		}
		else
		{
			$output['type'] 	= 'text';
			if (!is_null($this->getAttribute('maxLength'))) $output['maxLength'] = $this->getAttribute('maxLength');
			$output['size'] 		= $this->getAttribute('size');
		}
		$this->addOutputCode($output);
		
		if ($this->getAttribute('htmlEditor')===true)
		{
			$rootComponent = &$this->getRootComponent();
			
			if (!org_glizy_ObjectValues::get('org.glizy.JS.TinyMCE', 'add', false))
			{
				org_glizy_ObjectValues::set('org.glizy.JS.TinyMCE', 'add', true);
				
				$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('tiny_mce.js', 'tiny_mce/', false), 'head');
				$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('Glizy_tiny_mce.js'), 'head', true);
				
				$jsCode = 'Glizy.tinyMCE_plugins = "'.__Config::get( 'TINY_MCE_DEF_PLUGINS' ).'";';
				$jsCode .= 'Glizy.tinyMCE_btn1 = "'.__Config::get( 'TINY_MCE_BUTTONS1' ).'";';
				$jsCode .= 'Glizy.tinyMCE_btn2 = "'.__Config::get( 'TINY_MCE_BUTTONS2' ).'";';
				$jsCode .= 'Glizy.tinyMCE_btn3 = "'.__Config::get( 'TINY_MCE_BUTTONS3' ).'";';
				if ( __Config::get( 'TINY_MCE_PLUGINS' ) != '' )
				{
					$jsCode .= 'Glizy.tinyMCE_plugins .= ",'.__Config::get( 'TINY_MCE_PLUGINS' ).'";';	
				}
				$rootComponent->addOutputCode(org_glizy_helpers_JS::JScode( $jsCode ), 'head');
			}
			
			if (!is_null($this->getAttribute('adm:tinyMCEplugin')))
			{
				$pluginsNames = explode( ',', $this->getAttribute('adm:tinyMCEplugin') );
				$pluginsPaths = array();
				for( $i=0; $i < count( $pluginsNames ); $i++ )
				{
					$pos = strrpos( $pluginsNames[ $i ], "/" );
					if ( $pos !== false )
					{
						$pluginsPaths[] = '../../../../../../'.$pluginsNames[ $i ];
						$pluginsNames[ $i ] = substr( $pluginsNames[ $i ], $pos + 1 );
					}
					else
					{
						$pluginsPaths[] = $pluginsNames[ $i ];	
					}
				}
				if ( count( $pluginsPaths ) )
				{
					$jsCode = 'Glizy.tinyMCE_plugins += ",'.implode( ',', $pluginsPaths ).'";';
					$jsCode .= 'Glizy.tinyMCE_pluginsNames += ",'.implode( ',', $pluginsNames ).'";';
					$rootComponent->addOutputCode(org_glizy_helpers_JS::JScode( $jsCode ), 'head');
				}
			}
		}
	}
}
?>