<?php
/* SVN FILE: $Id$ */

/**
 * 
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.components
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */
 
glz_import('org.glizy.helpers.Navigation');

class org_glizy_components_LoginBox extends org_glizy_components_Component
{	
	var $_error = NULL;
	
	/**
	 * Return the component information
	 * 
	 * @return	array	component informaton.
	 * @access	public
	 * @static
	 */
	function getInfo()
	{
		$info 					= parent::info();
		$info['name']			= 'LoginBox';
		$info['class'] 			= 'org.glizy.components.LoginBox';
		$info['package'] 		= 'Glizy';
		$info['version'] 		= GLZ_CORE_VERSION;
		$info['author'] 		= 'Daniele Ugoletti';
		$info['author-email'] 	= 'daniele@ugoletti.com';
		$info['url'] 			= 'http://www.ugoletti.com';
		return $info;
	}
	
	/**
	 * Init
	 * 
	 * @return	void
	 * @access	public
	 */
	function init()
	{
		$this->defineAttribute('accessPageId',		false, 	NULL,						COMPONENT_TYPE_STRING);
		$this->defineAttribute('allowGroups',		false, 	'',							COMPONENT_TYPE_STRING);
		$this->defineAttribute('askPasswordLabel',	false, 	'',							COMPONENT_TYPE_STRING);
		$this->defineAttribute('askPasswordUrl',	false, 	'',							COMPONENT_TYPE_STRING);
		$this->defineAttribute('backend',			false, 	false,						COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('confirmLabel',		false, 	'',							COMPONENT_TYPE_STRING);
		$this->defineAttribute('cssClass',			false, 	'',							COMPONENT_TYPE_STRING);
		$this->defineAttribute('errorLabel',		false, 	__T('GLZ_LOGIN_ERROR'),		COMPONENT_TYPE_STRING);
		$this->defineAttribute('userLabel',			false, 	__T('GLZ_USER_LOGINID'), 	COMPONENT_TYPE_STRING);
		$this->defineAttribute('userField',			false, 	'loginuser', 				COMPONENT_TYPE_STRING);
		$this->defineAttribute('passwordLabel',		false, 	__T('GLZ_USER_PASSWORD'), 	COMPONENT_TYPE_STRING);
		$this->defineAttribute('passwordField',		false, 	'loginpsw', 				COMPONENT_TYPE_STRING);
		$this->defineAttribute('registrationUrl',	false, 	'', 						COMPONENT_TYPE_STRING);
		$this->defineAttribute('registrationLabel',	false, 	'', 						COMPONENT_TYPE_STRING);
		$this->defineAttribute('rememberLabel',		false, 	__T('GLZ_LOGIN_REMEMBER'), 	COMPONENT_TYPE_STRING);
		$this->defineAttribute('rememberField',		false, 	'remember', 				COMPONENT_TYPE_STRING);
		$this->defineAttribute('title',				false, 	'', 						COMPONENT_TYPE_STRING);
		
		parent::init();
	}	
	
	function process()
	{
		$error 		= false;
		$userName 	= trim(org_glizy_Request::get($this->getAttribute('userField'), $_COOKIE['glizy_username']));
		$psw 		= trim(org_glizy_Request::get($this->getAttribute('passwordField'),  $_COOKIE['glizy_password']));
		$remember	= org_glizy_Request::get($this->getAttribute('rememberField'), 0);

		if (!empty($userName) && !empty($psw))
		{
			org_glizy_Session::set('glizy.userLogged', false);
			org_glizy_Session::set('glizy.user', NULL);
		
			$psw = $_COOKIE['glizy_password']==$psw  ? $psw : glz_password($psw);
			$ar = &org_glizy_ObjectFactory::createModel('org.glizy.models.User');
			$result = $ar->find(array('user_loginId' => $userName, 'user_password' => $psw));
	
			if ($result)
			{
				$arGroup = &org_glizy_ObjectFactory::createModel('org.glizy.models.UserGroup');
				$arGroup->load($ar->user_FK_userGroup_id);
				
				$allowGroups = $this->getAttribute('allowGroups')!='' ? explode(',', $this->getAttribute('allowGroups')) : array();

				if ($ar->user_isActive==1 && ($this->getAttribute('backend') ? $arGroup->usergroup_backEndAccess : true) && (count($allowGroups) ? in_array($ar->user_FK_userGroup_id, $allowGroups) : true))
				{
					$lifetime = time() + 60*60*24*30;
					org_glizy_Session::set('glizy.userLogged', true);
					$acceptLogin = true;
					$logId = 0;
					if (org_glizy_Config::get('USER_LOG'))
					{
						// controlla se  gi loggato l'utente
						$arLog = &org_glizy_ObjectFactory::createModel('org.glizy.models.UserLog');
						// cancella i log scaduti
						$arLog->execSql( "DELETE FROM  ##TABLE_NAME## WHERE timediff( now(), userlog_lastAction) > '00:05:00'" );
						$arLog->userlog_FK_user_id 	= $ar->user_id;
						$r = $arLog->find();
						if ( $r )
						{
							// controlla se siamo sullo stesso IP
							if ( $arLog->userlog_ip == $_SERVER['REMOTE_ADDR'] &&  $_COOKIE['glizy_code'] == md5($arLog->userlog_session))
							{
								// stesso IP e stesso browser
								// il login viene accettato
								// cancella il log precedete
								$arLog->delete();
							}
							else
							{

								// computer differente
								// viene controllato se  passato il tempo
								if ( time() - strtotime(str_replace( "/", "-", $arLog->userlog_lastAction)) < 60 * 5 )
								{
									// non accetta il login
									$acceptLogin = false;
									$this->_content['errorLabel'] = org_glizy_locale_Locale::get('GLZ_LOGIN_USED');
								}
							}
						}
						
						if ( $acceptLogin )
						{
							$arLog = &org_glizy_ObjectFactory::createModel('org.glizy.models.UserLog');
							$arLog->userlog_FK_user_id 	= $ar->user_id;
							$arLog->userlog_session 	= org_glizy_Session::getSessionId();
							$arLog->userlog_ip 			= $_SERVER['REMOTE_ADDR'];
							$logId = $arLog->save();
							
							setcookie( "glizy_code", md5(org_glizy_Session::getSessionId()), $lifetime);
						}
					}
					
					if ( $acceptLogin )
					{
					$user = array(	'id' => $ar->user_id,
									'firstName' => $ar->user_firstName,
									'lastName' => $ar->user_lastName,
										'loginId' => $ar->user_loginId,
										'email' => $ar->user_email,
									'groupId' => $ar->user_FK_userGroup_id,
									'backEndAccess' => $arGroup->usergroup_backEndAccess,
									'language' => org_glizy_Request::get($this->getAttribute('languageField'), 'it'),
									'logId' => $logId);
					org_glizy_Session::set('glizy.user', $user);
					$this->setAttribute('visible', false);
					
					if ($remember)
					{
						setcookie( "glizy_username", $userName, $lifetime); //, "/", $_SERVER["HTTP_HOST"], !$_SERVER["HTTPS"]);
						setcookie( "glizy_password", $psw, $lifetime); //, "/", $_SERVER["HTTP_HOST"], !$_SERVER["HTTPS"]);
					}
					
					$user 				= $ar->getValuesAsArray();
					$user['userName'] 	= $userName;
					$user['psw'] 		= $psw;
					$evt = array('type' => GLZ_EVT_USERLOGIN, 'data' => $user);
					$this->dispatchEvent($evt);
					
					if ( $this->getAttribute('accessPageId') != $this->_application->getPageId() )
					{
						$destPage = is_null($this->getAttribute('accessPageId')) ? org_glizy_helpers_Link::scriptUrl() : org_glizy_helpers_Link::makeUrl('link', array('pageId' => $this->getAttribute('accessPageId')));					
						$url = 	org_glizy_Session::get('glizy.loginUrl', $destPage);
						org_glizy_helpers_Navigation::gotoUrl($url);
						exit();
					}
				}
				}
				else
				{
					$this->_content['errorLabel'] = org_glizy_locale_Locale::get('GLZ_LOGIN_DISABLED');
				}
			}
			else
			{
				$this->_content['errorLabel'] = $this->getAttribute('errorLabel');
			}
		}
		else
		{
			$this->_content['errorLabel'] = org_glizy_Session::get('glizy.loginError', NULL);
			org_glizy_Session::remove('glizy.loginError');
		}
		
		$this->_content['id'] 					= $this->getId();
		$this->_content['submitName'] 			= 'submit_'.$this->getId();
		$this->_content['cssClass'] 			= $this->getAttribute('cssClass');
		$this->_content['userLabel'] 			= $this->getAttribute('userLabel');
		$this->_content['userField'] 			= $this->getAttribute('userField');
		$this->_content['passwordLabel'] 		= $this->getAttribute('passwordLabel');
		$this->_content['passwordField'] 		= $this->getAttribute('passwordField');
		$this->_content['registrationPage'] 	= $this->getAttribute('registrationPage');
		$this->_content['registrationLabel'] 	= $this->getAttribute('registrationLabel');
		$this->_content['confirmLabel'] 		= $this->getAttribute('confirmLabel');
		$this->_content['rememberLabel'] 		= $this->getAttribute('rememberLabel');
		$this->_content['askPasswordLabel'] 	= $this->getAttribute('askPasswordLabel');
		$this->_content['title'] 				= $this->getAttributeString('title');
		$this->_content['__url__'] 				= org_glizy_helpers_Link::makeURL($this->getAttribute('registrationUrl'));
		$this->_content['askPasswordUrl'] 		= org_glizy_helpers_Link::makeURL($this->getAttribute('askPasswordUrl'));
	}
}

if (!class_exists('org_glizy_components_LoginBox_render'))
{
	class org_glizy_components_LoginBox_render extends org_glizy_components_render_Render
	{	
		function getDefaultSkin()
		{
			$skin = <<<EOD
<div tal:attributes="id LoginBox/id; class LoginBox/cssClass">
	<h3 tal:content="LoginBox/title" /> 
	<div>
		<form id="" method="post" action="" tal:attributes="id LoginBox/id">
			<p class="error" tal:condition="LoginBox/errorLabel" tal:content="structure LoginBox/errorLabel"></p>
			<label tal:attributes="for LoginBox/userField" tal:content="structure LoginBox/userLabel" /><br />
			<input type="text" class="text" tal:attributes="id LoginBox/userField; name LoginBox/userField"/><br />
			<label tal:attributes="for LoginBox/passwordField" tal:content="structure LoginBox/passwordLabel" /><br />
			<input type="password" class="text" tal:attributes="id LoginBox/passwordField; name LoginBox/passwordField"/><br />
			<table>
				<tr>
					<td><input name="remember" id="remember" value="1" type="checkbox" /><label  for="remember" tal:content="structure LoginBox/rememberLabel" /></td>
					<td class="submitButton"><input type="submit" class="submitButton" tal:attributes="name LoginBox/submitName;structure value LoginBox/confirmLabel"/></td>
				</tr>
			</table>
			<a class="link" tal:attributes="href LoginBox/__url__" tal:content="structure LoginBox/registrationLabel"></a><br />
			<a class="link" tal:attributes="href LoginBox/askPasswordUrl" tal:content="structure LoginBox/askPasswordLabel"></a>
		</form>
	</div>
</div>
EOD;
			return $skin;
		}
	}
}
?>