<?php
/* SVN FILE: $Id$ */

/**
 * 
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.components
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */
 
glz_import('org.glizy.components.Text');
glz_import('org.glizy.helpers.Html');

class org_glizy_components_LongText extends org_glizy_components_Text
{
/**
	 * Return the component information
	 * 
	 * @return	array	component informaton.
	 * @access	public
	 * @static
	 */
	function getInfo()
	{
		$info 					= parent::info();
		$info['name']			= 'LongText';
		$info['class'] 			= 'org.glizy.components.LongText';
		$info['package'] 		= 'Glizy';
		$info['version'] 		= GLZ_CORE_VERSION;
		$info['author'] 		= 'Daniele Ugoletti';
		$info['author-email'] 	= 'daniele@ugoletti.com';
		$info['url'] 			= 'http://www.ugoletti.com';
		return $info;
	}

	/**
	 * Init
	 * 
	 * @return	void
	 * @access	public
	 */
	function init()
	{
		// define the custom attributes
		$this->defineAttribute('label',					false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('forceP',				false, 	true,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('paginate',				false, 	NULL,	COMPONENT_TYPE_OBJECT);	
		$this->defineAttribute('adm:cols',				false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('adm:htmlEditor',		false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('adm:maxLength',			false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('adm:required',			false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('model:required',		false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('adm:requiredMessage',	false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('adm:rows',				false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('adm:tinyMCEplugin',		false, 	NULL,	COMPONENT_TYPE_STRING);

		// call the superclass for validate the attributes
		parent::init();
	}


	/**
	 * Process
	 * 
	 * @return	boolean	false if the process is aborted
	 * @access	public
	 */
	function process()
	{
		parent::process();
		if ($this->getAttribute('adm:htmlEditor')!==true)
		{
			//$tagContent = strip_tags($this->getText());
			$tagContent = $this->getText();
			$this->setText($tagContent);
		}
		else
		{
			$tagContent = trim($this->getText());
			$tagContent = str_replace(array("\r\n", "\n", "\r"), "", $tagContent);
			$this->setText($tagContent);
		}
	}
	
	function render_html()
	{
		$this->addOutputCode($this->getContent());
	}
		
	
	function getContent()
	{
		$output = $this->getText();
		if ($this->getAttribute('adm:htmlEditor')!==true) 
		{
		
			$output = $this->encodePlainText();
			if ($this->getAttribute('forceP'))
			{
				$output = org_glizy_helpers_Html::forceP($output);
			}
			return $output;
		}
		else
		{
			$output = javascript_to_html( $output );
			// if ($this->getAttribute('forceP'))
			// {
			// 	$output = org_glizy_helpers_Html::forceP($output);
			// }

			// esegue la paginazione
			if (!is_null($this->getAttribute("paginate")))
			{
				$paginateClass 	= &$this->getAttribute("paginate");
				$this->_content = $paginateClass->splitTextResult($output);
			}
			
			$output = org_glizy_helpers_Link::parseInternalLinks($output);

			$attributes = array();
			$attributes 				= array();
			$attributes['id'] 			= $this->getId();
			$attributes['class']		= $this->getAttribute('cssClass');
			$wrapTag = $this->getAttribute('wrapTag');
			$outputTag .= !empty($wrapTag) ? '<'.$wrapTag.' '.$this->_renderAttributes($attributes).'>' : '';
			$outputTag .= $output;
			$outputTag .= !empty($wrapTag) ? '</'.$wrapTag.'>' : '';
			return $outputTag;
		}
	}
	
	
	function render_jsediting()
	{
		$output = array();
		$output['id'] 		= $this->getId();
		$output['label'] 	= $this->getAttribute('label');
		if ($this->getAttribute('adm:htmlEditor')===true)
		{
			$output['value'] = str_replace(array("\r\n", "\n", "\r"), "", javascript_to_html( $this->getText() ) );
			//$output['value'] = str_replace(array("\r\n", "\n", "\r"), "", rawurlencode( $this->getText() ) );
		}
		else
		{
			$output['value'] 	= str_replace( array("\r\n", "\n", "\r"), "<br />", $this->getText() );
		}
		$output['type'] 	= 'longtext';
		$output['htmlEditor'] 	= $this->getAttribute('adm:htmlEditor')===true ? 1 : 0;

		if ($this->getAttribute('adm:cols')!='') $output['cols'] = $this->getAttribute('adm:cols');
		if ($this->getAttribute('adm:rows')!='') $output['rows'] = $this->getAttribute('adm:rows');			
	
		if ($this->getAttribute('adm:required') || $this->getAttribute('model:required'))
		{
			$output['required'] = 'Glizy.validateForm';
			if (!is_null($this->getAttribute('adm:requiredMessage')))
			{
				$output['requiredMessage'] = $this->getAttribute('adm:requiredMessage');
			}
		}
		
		if (!$this->getAttribute('adm:htmlEditor'))
		{
			if (!is_null($this->getAttribute('adm:maxLength'))) $output['maxLength'] = $this->getAttribute('adm:maxLength');
		}
	
		$this->addOutputCode($output);
		
		if ($this->getAttribute('adm:htmlEditor')===true)
		{
			$rootComponent = &$this->getRootComponent();
			
			if (!org_glizy_ObjectValues::get('org.glizy.JS.TinyMCE', 'add', false))
			{
				org_glizy_ObjectValues::set('org.glizy.JS.TinyMCE', 'add', true);
				
				$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('tiny_mce.js?v='.GLZ_CORE_VERSION, 'tiny_mce/', false), 'head');
				$rootComponent->addOutputCode(org_glizy_helpers_JS::linkCoreJSfile('Glizy_tiny_mce.js?v='.GLZ_CORE_VERSION), 'head', true);
				
				$jsCode = 'Glizy.tinyMCE_plugins = "'.__Config::get( 'TINY_MCE_DEF_PLUGINS' ).'";';
				$jsCode .= 'Glizy.tinyMCE_btn1 = "'.__Config::get( 'TINY_MCE_BUTTONS1' ).'";';
				$jsCode .= 'Glizy.tinyMCE_btn2 = "'.__Config::get( 'TINY_MCE_BUTTONS2' ).'";';
				$jsCode .= 'Glizy.tinyMCE_btn3 = "'.__Config::get( 'TINY_MCE_BUTTONS3' ).'";';
				if ( __Config::get( 'TINY_MCE_PLUGINS' ) != '' )
				{
					$jsCode .= 'Glizy.tinyMCE_plugins .= ",'.__Config::get( 'TINY_MCE_PLUGINS' ).'";';	
				}
				$rootComponent->addOutputCode(org_glizy_helpers_JS::JScode( $jsCode ), 'head');
			}
			
		if (!is_null($this->getAttribute('adm:tinyMCEplugin')))
		{
			$pluginsNames = explode( ',', $this->getAttribute('adm:tinyMCEplugin') );
			$pluginsPaths = array();
			for( $i=0; $i < count( $pluginsNames ); $i++ )
			{
				$pos = strrpos( $pluginsNames[ $i ], "/" );
				if ( $pos !== false )
				{
					$pluginsPaths[] = '../../../../../../'.$pluginsNames[ $i ];
					$pluginsNames[ $i ] = substr( $pluginsNames[ $i ], $pos + 1 );
				}
				else
				{
					$pluginsPaths[] = $pluginsNames[ $i ];	
				}
			}
				if ( count( $pluginsPaths ) )
				{
					$jsCode = 'Glizy.tinyMCE_plugins += ",'.implode( ',', $pluginsPaths ).'";';
					$jsCode .= 'Glizy.tinyMCE_pluginsNames += ",'.implode( ',', $pluginsNames ).'";';
			$rootComponent->addOutputCode(org_glizy_helpers_JS::JScode( $jsCode ), 'head');
		}
			}
		}
	}
	
	function encodePlainText()
	{
		return str_replace( array("\r\n", "\n", "\r"), "<br />", glz_encodeOutput( str_replace( '<br />', "\r\n", $this->getText() ) ) );
	}
}
?>
