<?php
/* SVN FILE: $Id$ */

/**
 * 
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.components
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 core class
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */
 
glz_import('org.glizy.components.HtmlFormElement');

class org_glizy_components_RadioButton extends org_glizy_components_HtmlFormElement
{
	/**
	 * Return the component information
	 * 
	 * @return	array	component informaton.
	 * @access	public
	 * @static
	 */
	function getInfo()
	{
		$info 					= parent::info();
		$info['name']			= 'Checkbox';
		$info['class'] 			= 'org.glizy.components.Checkbox';
		$info['package'] 		= 'Glizy';
		$info['version'] 		= GLZ_CORE_VERSION;
		$info['author'] 		= 'Daniele Ugoletti';
		$info['author-email'] 	= 'daniele@ugoletti.com';
		$info['url'] 			= 'http://www.ugoletti.com';
		return $info;
	}
	
	/**
	 * Init
	 * 
	 * @return	void
	 * @access	public
	 */
	function init()
	{
		// define the custom attributes
		$this->defineAttribute('bindTo',			false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('cssClass',			false, 	'',		COMPONENT_TYPE_STRING);
		$this->defineAttribute('disabled',			false, 	NULL,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('label',				false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('labelPosition',		false, 	'left',	COMPONENT_TYPE_STRING);
		$this->defineAttribute('required',			false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('requiredMessage',	false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('value',				false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('defaultValue',		false, 	'0',	COMPONENT_TYPE_STRING);
		$this->defineAttribute('wrapLabel',			false, 	false,	COMPONENT_TYPE_BOOLEAN);
		$this->defineAttribute('name',				false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('title',				false, 	NULL,	COMPONENT_TYPE_STRING);
		$this->defineAttribute('checkedValue',		false, 	'1',	COMPONENT_TYPE_STRING);
	
		// call the superclass for validate the attributes
		parent::init();
	}
	
	/**
	 * Process
	 * 
	 * @return	boolean	false if the process is aborted
	 * @access	public
	 */
	function process()
	{
		$this->_content = $this->getAttribute('value');
		if (is_object($this->_content))
		{
			$contentSource = &$this->getAttribute('value');
			$this->_content = $contentSource->loadContent($this->getId(), $this->getAttribute('bindTo'));
		}
		else if (is_null($this->_content))
		{
			// richiede il contenuto al padre
			$this->_content = $this->_parent->loadContent($this->getId(), $this->getAttribute('bindTo'));
		}
		
		if (is_null($this->_content) || $this->_content=='') 
		{
			// imposta il valore di default
			$this->_content = $this->getAttribute('defaultValue');
		}
	}
	
	function render_html()
	{
		$attributes = $this->getContent(true);
		$output  = '<input '.$this->_renderAttributes($attributes).'/>';

		if ($this->getAttribute('labelPosition')=='left')
		{
			$output = org_glizy_helpers_Html::label($this->getAttributeString('label'), $this->getId(),  $this->getAttribute('wrapLabel'), $output, array( 'class' => $attributes['class'] ), false);
		}
		else
		{
			$this->_rightLabel = true;
			$output .= org_glizy_helpers_Html::label($this->getAttributeString('label'), $this->getId(),  $this->getAttribute('wrapLabel'), '', array( 'class' => $attributes['class'] ), false);
		}
		$this->addOutputCode($output);
	}
	
	function getContent($internal=false)
	{
		$name = $this->getAttribute( 'name' );
		$attributes 				= array();
		$attributes['id'] 			= $this->getId();
		$attributes['name'] 		= empty( $name ) ? $this->getOriginalId() : $name ;
		$attributes['class'] 		= $this->getAttribute('required') ? 'required' : '';
		$attributes['class'] 		.= $this->getAttribute( 'cssClass' ) != '' ? ( $attributes['class'] != '' ? ' ' : '' ).$this->getAttribute( 'cssClass' ) : '';
		$attributes['type'] 		= 'radio';
		$attributes['disabled'] 	= $this->getAttribute('readOnly') ? 'readonly' : '';

		$attributes['value'] 		= $this->getAttribute( 'checkedValue' );
		if ( $attributes['value'] == "" ) $attributes['value'] = $attributes['id'];
		$attributes['checked'] 		= $this->_content == $attributes['value'] ? 'checked' : '';
		if (!$internal) $attributes['label'] = $this->encodeOuput($this->getAttribute('label'));
		$attributes['title'] = $this->getAttributeString('title');
		return $attributes;
	}
	
	function render_jsediting()
	{
		$output = array();
		$output['id'] 		= $this->getId();
		$output['type'] 	= 'radio';
		$output['formName'] 	= $this->getAttribute('name');
		$output['label'] 	= $this->getAttribute('label');
		$output['value'] 	= $this->getAttribute( 'checkedValue' ) == $this->_content ? '1' : '0';
		if ($this->getAttribute('required'))
		{
			$output['required'] = 'Glizy.validateForm';
			if (!is_null($this->getAttribute('requiredMessage')))
			{
				$output['requiredMessage'] = $this->getAttribute('requiredMessage');
			}
		}
		$this->addOutputCode($output);
	}
}
?>