<?php
//mysql:dbname=testdb;host=127.0.0.1
class Connection extends PDO
{
	public $debug = false;
	private $_errorCode = 0;
	private $_errorInfo = "";
	private $_dsn;
	
	function __construct($dsn, $username, $password, $driver_options = array() )
	{
		$this->_dsn = $dsn;
		parent::__construct($dsn, $username, $password, $driver_options);
	}

	function execute( $sql, $param = array(), $silent = false )
	{
		if ( $this->debug ) $time_start = microtime();
		
		if ( strpos( $this->_dsn, 'sqlite:' ) !== false )
		{
			preg_match_all( '/(CONCAT\(([^\)]*)\))/', $sql, $match );
			if ( count( $match[ 0 ] ) )
			{
				$sql = str_replace( $match[ 0 ], str_replace( ',', ' || ', $match[ 2 ]), $sql );
			}
		}
		$sth = $this->prepare( $sql );
		$r = $sth->execute( $param );
		$this->_errorCode = $sth->errorCode();
		$this->_errorInfo = $sth->errorInfo();
		
		if ( $this->debug )
		{
			$time_end = microtime();
			$time = $time_end - $time_start;
			echo "[time: ".$time.", ".( $r ? "records: ".$sth->rowCount() : "" )."] ";
			if ( count( $param ) )
			{
				echo vsprintf( str_replace( "?", "'%s'", $sql ), $param );
			}
			else
			{
				echo $sql;	
			}
			echo "<br />\n\r";
		}

		if ( !$r )
		{
			if ( !$silent ) echo $this->ErrorMsg()."<br />\n\r";
			return false;
		}
		return new DBResultSet( $sth, $this, strpos( $this->_dsn, 'sqlite:' ) !== false );
	}
	

	
	function qstr( $str, $option )
	{
		return '\''.addslashes( $str ).'\'';
	}
	
	function SelectDB( $n )
	{
	}
		
	function selectLimit( $sql, $limitLength, $limitStart, $params = array() )
	{
		return $this->execute( $sql.' LIMIT '.$limitStart.', '.$limitLength, $params );
	}
	
	function Insert_ID()
	{
		return $this->lastInsertId();
	}
	
	function ErrorNo()
	{
		if ( $this->_errorCode != 0 )
		{
			return $this->_errorCode ;
		}
		else
		{
			return false;
		}
	}
	
	function ErrorMsg()
	{
		$info = $this->_errorInfo;
		return $info[2];
	}
	
	function MetaColumns( $table, $param )
	{
		if ( strpos( $this->_dsn, 'sqlite:' ) !== false )
		{
			$sql = 'PRAGMA table_info ('.$table.')';
			$stmt = $this->prepare($sql);
			
			try
			{    
				if( $stmt->execute() )
				{
					$meta = array();
					$raw_column_data = $stmt->fetchAll();

					foreach($raw_column_data as $outer_key => $array)
					{
						$dbField = new DBfield;
						$dbField->name = $array['name'];
						$dbField->type = $array['type'];
						$dbField->max_length = -1;
						$dbField->not_null = $array['notnull'] == '1';
						$dbField->has_default = $array['dflt_value'] != null;
						$dbField->default_value = $array['dflt_value'];
						$dbField->primary_key = $array['pk'] == '1';
						$dbField->auto_increment = false;
						$meta[ $array['name'] ] = $dbField;
					}        
				}

				return $meta;
			} catch (Exception $e){
				return $e->getMessage(); //return exception
			}
		}
		else
		{
		$sql = 'SHOW COLUMNS FROM ' . $table;
		$stmt = $this->prepare($sql);
			if( $stmt->execute() )
			{
				$meta = array();
				$raw_column_data = $stmt->fetchAll();

				foreach($raw_column_data as $outer_key => $array)
				{
					$dbField = new DBfield;
					$dbField->name = $array['Field'];
					if ( preg_match('/^enum/i', $array['Type'] ) )
					{
						$dbField->type = $array['Type'];
						
					}
					else
					{
						$dbField->type = preg_replace( '/([^\(]*).*/', '$1', $array['Type'] );
					}
					$dbField->max_length = preg_replace( '/([^\(]*)\((\d*)\).*/', '$2', $array['Type'] );
					if ( $dbField->max_length == $array['Type'] ) $dbField->max_length = -1;
					$dbField->not_null = $array['Null'] == 'NO';
					$dbField->has_default = $array['Default'] != '';
					$dbField->default_value = $array['Default'];
					$dbField->primary_key = $array['Key'] == 'PRI';
					$dbField->auto_increment = $array['Extra'] == 'auto_increment';
					$meta[ $array['Field'] ] = $dbField;
				}        
			}

			return $meta;
		}
	}
}

class DBResultSet
{
	public $fields;
	public $_currentRow;
	public $EOF;
	private $sth;
	private $_count;
	private $_sqlite;
	private $_conn;
	private $buffer;
	private $pos;

	function __construct( $sth, $conn, $sqlite )
	{
		$this->_conn = $conn;
		$this->_sqlite = $sqlite;
		$this->sth = $sth;
		$this->buffer = array();
		$this->pos = 0;
		$this->fetch();
	}
	
	function MoveFirst()
	{
		$this->pos = 0;
		$this->fetch();
	}
	
	
	function MoveNext()
	{
		$this->fetch();
	}
	
	function RecordCount()
	{
		if ( $this->_sqlite )
		{
			if ( is_null( $this->_count ) )
			{
				$sql = 'select count(*) from ( '.$this->sth->queryString.' )';
				$sth = $this->_conn->prepare( $sql );
				$r = $sth->execute( $param );
				$row = $sth->fetch(PDO::FETCH_NUM);
				$this->_count = $row[ 0 ];
			}
			return $this->_count;
		}
		else
		{
		return $this->sth->rowCount();
		}
	}
	
	private function fetch()
	{
		if (  isset( $this->buffer[ $this->pos ] ) )
		{
			$this->fields = $this->buffer[ $this->pos ];
			$this->_currentRow = $this->fields;	
			$this->EOF = count( $this->buffer ) === $this->pos;		
		}
		else
		{
			$this->fields = $this->sth->fetch(PDO::FETCH_ASSOC);
			$this->_currentRow = $this->fields;	
			$this->EOF = $this->_currentRow === false;			
			if ( !$this->EOF )
			{
				$this->buffer[ $this->pos ] = $this->fields;
			}
		}

		$this->pos++;
	}
}

class DBfield
{
	public $name;
	public $max_length;
	public $type;
	public $not_null;
	public $has_default;
	public $default_value;
	public $scale;
	public $primary_key;
	public $auto_increment;
	public $binary = false;
	public $enums;
}
?>
