<?php
/* SVN FILE: $Id$ */

/**
 *
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 *
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   glizy.helpers
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 script
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */

glz_import('org.glizy.helpers.Html');
 
class org_glizy_helpers_Link extends GlizyObject
{
	function getDefaultUrl()
	{
		return GLZ_SCRIPNAME;
	}
	
	function makeURL($route='', $queryVars=array(), $addParam=array())
	{
		return __Routing::makeURL( $route, $queryVars, $addParam );
	}
	

	function makeLink($route='', $queryVars=array(), $addParam=array(), $onclick='', $encode = true )
	{
		$cssClass = '';
		if (isset($queryVars['cssClass']))
		{
			$cssClass = $queryVars['cssClass'];
			unset($queryVars['cssClass']);
		}
		$rel = '';
		if (isset($queryVars['rel']))
		{
			$rel = $queryVars['rel'];
			unset($queryVars['rel']);
		}
		$label = isset($queryVars['label']) ? $queryVars['label'] : $queryVars['title'];
		unset( $queryVars['label'] );
		$url = org_glizy_helpers_Link::makeURL($route, $queryVars, $addParam);
		return org_glizy_helpers_Html::renderTag(	'a',
													array(	'href' => $url,
															'class' => $cssClass,
															'title' => glz_encodeOutput( $queryVars['title'] ),
															'rel' => $rel,
															'onclick' => $onclick),
													true,
													$encode ? glz_encodeOutput( $label ) : $label );
	}

	
	function makeLink2($route, $options=array(), $attributes=array())
	{
		if (isset($options['id']))
		{
			$attributes['id'] = $options['id'];
		}
		$attributes['title'] 	= glz_encodeOutput(isset($options['label']) ? $options['label'] : $options['title']);
		$attributes['href'] 	= isset($options['url']) ? $options['url'] : org_glizy_helpers_Link::makeURL($route, $options);

		$output = org_glizy_helpers_Html::renderTag('a', $attributes, true, $attributes['title']);
		return $output;
	}

	function makeJSLocation($route='', $queryVars=array(), $addParam=array())
	{
		$url = org_glizy_helpers_Link::makeURL($route, $queryVars, $addParam);
		return 'location.href = \''.$url.'\'';
	}

	

	function makeSimpleLink($label, $url, $title='', $cssClass='', $rel='')
	{
		if (empty($title)) $title = $label;
		return org_glizy_helpers_Html::renderTag(	'a',
													array(	'href' => $url,
															'class' => $cssClass,
															'title' => $title,
															'rel' => $rel),
													true,
													$label);
	}

	function makeEmailLink($email, $label=NULL)
	{
		if (empty($label)) $label = $email;
		return '<a href="mailto:'.$email.'" title="'.glz_encodeOutput($email).'">'.glz_encodeOutput($label).'</a>';
	}

	function imageLink($route, $options=array(), $attributes=array())
	{
		$options['title'] 	= glz_encodeOutput($options['title']);
		$attributes['title'] = $options['title'];
		$attributes['alt'] 	= $options['title'];
		$attributes['src'] 	= $options['src'];
		$attributes['width'] 	= @$options['width'];
		$attributes['height'] 	= @$options['height'];
		unset($options['src']);
		$url = isset($options['url']) ? $options['url'] : org_glizy_helpers_Link::makeURL($route, $options);
	
		$output  = org_glizy_helpers_Html::renderTag('a',
													array(	'href' => $url,
															'class' => $options['cssClass'],
															'title' => $options['title'],
															'rel' => $rel),
													true,
													org_glizy_helpers_Html::renderTag('img', $attributes));
		return $output;
	}

	function assetsLink( $iconName, $title, $routing, $params, $confirm='' )
	{
		$params[ 'label' ] = __Assets::getIcon( $iconName, $title );
		$params[ 'title' ] = $title;		
		return org_glizy_helpers_Link::makeLink( $routing, $params, array(), $confirm );
	}
	

	function addParams($params=array(), $absolute=false)
	{
		$url = __Routing::$queryString;
		if (count($params))
		{
			foreach($params as $k=>$v)
			{
				if (preg_match('/'.$k.'=/', $url))
				{
					$url = preg_replace('/('.$k.'=)([^\&]*)/', $k.'='.$v, $url);
				}
				else
				{
					$url .= '&'.$k.'='.$v;
				}

			}
		}
		$url = preg_replace('/&(?!amp;)/i', '&amp;', $url);
		return __Routing::scriptUrlWithParams( $url, $absolute );
	}

	function addParamsJS( $params=array() )
	{
		$url = org_glizy_helpers_Link::addParams( $params, true );
		return 'location.href = \''.$url.'\'';
	}

	
	function removeParams($params=array(), $url = null)
	{
		if ( is_null( $url ) )
		{
			$url = __Routing::$queryString;
		}

		if (count($params))
		{
			foreach($params as $v)
			{
				if (preg_match('/('.$v.'=)([^\&]*)(\&?)/', $url))
				{
					$url = preg_replace('/('.$v.'=)([^\&]*)(\&?)/', '', $url);
				}
			}
		}
		$url = trim($url, '&');
		return __Routing::scriptUrlWithParams( $url );
	}

	function removeParamsJS( $params=array() )
	{
		$url = org_glizy_helpers_Link::removeParams( $params, true );
		return 'location.href = \''.$url.'\'';
	}

	function makeLanguageSwitch($language, $languageCode)
	{
		// TODO sef e routing
		$scriptUrl = $_SERVER["QUERY_STRING"];
		if (empty($scriptUrl)) $scriptUrl = org_glizy_Paths::get('PAGE_INDEX').'?'.$languageCode.'/1/home';
		else $scriptUrl = org_glizy_Paths::get('PAGE_INDEX').'?'.preg_replace('/([^\/]*)(\/.*)$/', $languageCode.'$2', $scriptUrl );
		return org_glizy_helpers_Link::makeSimpleLink($language, $scriptUrl);
	}



	function scriptUrl($removeGetParams=false)
	{
		if (!$removeGetParams)
		{
			return  org_glizy_Paths::get('PAGE_INDEX').'?'.$_SERVER["QUERY_STRING"];
		}
		else
		{
			list($url) = explode('&', $_SERVER["QUERY_STRING"]);
			return  org_glizy_Paths::get('PAGE_INDEX').'?'.$url;
		}
	}
	
	function getUrlFromRequest()
	{
		$requestValues = __Request::getAllAsArray();
		$url = array();

		foreach ( $requestValues as $k=>$v )
		{
			if ( empty( $v ) ) continue;

			if ( is_array( $v ) )
			{				
				for( $i = 0; $i < count( $v ); $i++ )
				{
					$url[] = $k.'[]='.$v[ $i ];
				}
			}
			else
			{
				$url[] = $k.'='.$v;
			}
		}
		return org_glizy_Paths::get('PAGE_INDEX').'?'.implode( '&', $url );;
	}


	function parseInternalLinks($text, $absolute = false )
	{
		$serverUrl = $absolute ? GLZ_HOST_ROOT.'/' : '';
		preg_match_all('/<a.*href=["\'](internal\:)(\d*)["\'].*/Ui', $text, $internalLinks);
		if (count($internalLinks) && count($internalLinks[0]))
		{
			for ($i=0; $i<count($internalLinks[0]); $i++)
			{
				$link = $serverUrl.org_glizy_helpers_Link::makeURL('link', array('pageId' => $internalLinks[2][$i]));
				$originaLink = $internalLinks[0][$i];
				$newLink = str_replace('internal:'.$internalLinks[2][$i], $link, $originaLink);
				$text = str_replace($originaLink, $newLink, $text);
			}
		}

		preg_match_all('/<a.*href=["\'](glossary\:)(.*)["\'].*/Ui', $text, $internalLinks);
		if (count($internalLinks) && count($internalLinks[0]))
		{
			for ($i=0; $i<count($internalLinks[0]); $i++)
			{
				$link = $serverUrl.org_glizy_helpers_Link::makeURL('glossary', array('glossarydetail_term2' => '' )).urlencode($internalLinks[2][$i] );
				$originaLink = $internalLinks[0][$i];
				$newLink = str_replace('glossary:'.$internalLinks[2][$i], $link, $originaLink);
				$text = str_replace($originaLink, $newLink, $text);
			}
		}

		preg_match_all('/<a.*href=["\'](media\:)(.*)(\:)(.*)["\'].*/Ui', $text, $internalLinks);
		if (count($internalLinks) && count($internalLinks[0]))
		{
			for ($i=0; $i<count($internalLinks[0]); $i++)
			{
				$link = $serverUrl.'getFile.php?id='.$internalLinks[2][$i];
				$originaLink = $internalLinks[0][$i];
				$newLink = str_replace('media:'.$internalLinks[2][$i].":".$internalLinks[4][$i], $link, $originaLink);
				$text = str_replace($originaLink, $newLink, $text);
			}
		}

		// non è il massimo ma la regexp su testi lunghi crasha
		$text = str_replace( array( 'href="#', 'href=\'#' ), array( 'href="'.__Routing::scriptUrl().'#', 'href=\''.__Routing::scriptUrl().'#' ), $text );
		// $text = preg_replace("/<(.*?)(href)\s*=\s*(\'|\")#(.*?)(\'|\")(.*?)>/si", "<$1$2=$3".__Routing::scriptUrl()."#$4$5$6>", $text);


		if ( $absolute )
		{
			$text = str_replace( 'src="getImage', 'src="'.$serverUrl.'getImage', $text);	
			$text = str_replace( 'src="cache/', 'src="'.$serverUrl.'cache/', $text);	
		}
		return $text;
	}



	function formatLink($link, $label=NULL, $content=NULL, $cssClass='')
	{
		if (is_null($label)) $label = $link;
		if (is_null($content)) $content = $label;
		if (!is_null($cssClass)) $cssClass = ' class="'.$cssClass.'"';
		$link = preg_replace("/^http:\/\//", "", $link);
		$link = preg_replace('/&(?!amp;)/i', '&amp;', $link);
		$protocol = ((strpos($link, "index.php")==0 && strpos($link, "index.php")!==false) || (strpos($link, "?")==0 && strpos($link, "?")!==false)) ? "":"http://";
		$target = ((strpos($link, "index.php")==0 && strpos($link, "index.php")!==false) || (strpos($link, "?")==0 && strpos($link, "?")!==false)) ? "internal":"external";

		
		$link = '<a href="'.$protocol.$link.'" rel="'.$target.'" title="'.$label.'"'.$cssClass.'>'.$content.'</a>';
		return $link;
	}
	
	function formatInternalLink($link, $label=NULL, $content=NULL, $cssClass='', $target='internal')
	{
		if (is_null($label)) $label = $link;
		if (is_null($content)) $content = $label;
		if (!is_null($cssClass)) $cssClass = ' class="'.$cssClass.'"';
			
		$link = '<a href="'.$link.'" rel="'.$target.'" title="'.$label.'"'.$cssClass.'>'.$content.'</a>';
		return $link;
	}
}

class __Link extends org_glizy_helpers_Link
{
}
?>