<?php
/* SVN FILE: $Id$ */

/**
 * Glizy global functions definition.
 *
 * This file is part of the GLIZY framework.
 * Copyright (c) 2005-2006 Daniele Ugoletti <daniele@ugoletti.com>
 * 
 * For the full copyright and license information, please view the COPYRIGHT.txt
 * file that was distributed with this source code.
 *
 * @copyright    Copyright (c) 2005, 2006 Daniele Ugoletti
 * @link         http://www.glizy.org Glizy Project
 * @license      http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 * @package      glizy
 * @subpackage   core
 * @author		 Daniele Ugoletti <daniele@ugoletti.com>
 * @category	 script
 * @since        Glizy v 0.01
 * @version      $Rev$
 * @modifiedby   $LastChangedBy$
 * @lastmodified $Date$
 */


function glz_require_once_dir($dir, $default='')
{
	$dir = rtrim($dir, "*");
	if ($default!='')
	{
		if (!is_array($default))
		{	
			$default = array($default);
		}
		
		foreach ($default as $value)
		{
			$retArray[] = "$dir/$value";
			require_once("$dir/$value");
		}					
	}
	
	if ($dir_handle = @opendir($dir))
	{
		while ($file_name = readdir($dir_handle))
		{			
			if ($file_name!="." &&
				$file_name!=".." &&
				$file_name!=$default &&
				!is_dir("$dir/$file_name") &&
				substr($file_name, -3)=='php' &&
				strpos($file_name, '._') === false )
			{
				require_once("$dir/$file_name");
			}
		}
		closedir($dir_handle);
	}
	else
	{
		echo "Could not open directory $dir";
	}	
}



function glz_import($classPath, $classToReadFirst=array(), $path='')
{
	static $loadedClass = array();
	$classPath 			= str_replace('.', '/', $classPath);
	$classPath 			= rtrim($classPath, '*');
	$origClassPath 		= $classPath;

	if (in_array($classPath, $loadedClass)) return true;

	if (empty($path))
	{
		if (class_exists('org_glizy_Paths') && !is_null(org_glizy_Paths::get('APPLICATION_CLASSES')))
		{
			$path = NULL;
			$searchPath = org_glizy_Paths::getClassSearchPath();
			foreach($searchPath as $p)
			{
				$path = $p.$classPath;
				if (file_exists($p.$classPath) || file_exists($p.$classPath.'.php'))
				{
					$path = $p;
					break;
				}
			}

			if (is_null($path))
			{
				// TODO
				// visualizzare errore
				echo "errore";
			}
		}
		else
		{
			$path = realpath(dirname(__FILE__)).'/classes/';
		}
	}
	
	if (substr($classPath, -1, 1)=='/' || $classPath=='')
	{
		// import all file in the folder
		$classPath = rtrim($classPath, '/');
		glz_require_once_dir($path.$classPath, $classToReadFirst);
		glz_loadLocale($path.$classPath);
	}
	else
	{
		// import a single file
		if (file_exists($path.$classPath.'.php'))
		{
			require_once($path.$classPath.'.php');
		}
		else
		{
			return false;
		}
	}

	$loadedClass[] = $origClassPath;
	return true;
}

function glz_loadLocale($path)
{
	if (class_exists('org_glizy_ObjectValues'))
	{
		$application = &org_glizy_ObjectValues::get('org.glizy', 'application');
		// check if it's available a locale file
		if (is_object($application) && file_exists($path.'/locale/'.$application->getLanguage().'.php'))
		{
			require_once($path.'/locale/'.$application->getLanguage().'.php');
		}
		else if (is_object($application) && file_exists($path.'/locale/en.php'))
		{
			require_once($path.'/locale/en.php');
		}
	}
}

function glz_findClassPath($classPath)
{
	if (!class_exists('org_glizy_Paths')) return NULL;
	$classPath = str_replace(array('.', '*'), '/', $classPath);

	$path = NULL;
	$searchPath = org_glizy_Paths::getClassSearchPath();
	foreach($searchPath as $p)
	{
		if (file_exists($p.$classPath))
		{
			$path = $p.$classPath;
			break;
		}
		else if (file_exists($p.$classPath.'.xml'))
		{
			$path = $p.$classPath.'.xml';
			break;
		}
		else if (file_exists($p.$classPath.'.php'))
		{
			$path = $p.$classPath.'.php';
			break;
		}
	}
	return $path;
}

function glz_importLib($path)
{
	if (!class_exists('org_glizy_Paths')) return NULL;
	require_once(org_glizy_Paths::get('CORE_LIBS').$path);
}

function glz_importApplicationLib($path)
{
	if (!class_exists('org_glizy_Paths')) return NULL;
	require_once(org_glizy_Paths::get('APPLICATION_LIBS').$path);
}

function glz_encodeOutput($output)
{
	if (is_array($output))
	{
		return glz_encodeOutputArray($output);
	}
	else return glz_htmlentities( javascript_to_html( $output ) );
}

function glz_encodeOutputArray($output)
{
	$keys = array_keys($output);
	$count = count($output);
	for ($i = 0; $i < $count; $i++)
	{
		if (is_array($output[$keys[$i]]))
		{
			$output[$keys[$i]] = glz_encodeOutputArray($output[$keys[$i]]);
		}
		else
		{
			$output[$keys[$i]] = glz_htmlentities(javascript_to_html($output[$keys[$i]]));
		}
	}
	return $output;
}

function glz_htmlentities( $text )
{
	$text = htmlentities( $text, ENT_COMPAT, GLZ_CHARSET );
	$badwordchars=array( chr(145), chr(146), chr(147), chr(148), chr(151) );
	$fixedwordchars=array( "&lsquo;", "&rsquo;", '&ldquo;', '&rdquo;', '&mdash;' );
	$text = str_replace($badwordchars,$fixedwordchars,$text);
	
	return str_replace( '&amp;#', '&#', $text );
}

function glz_password($psw)
{
	switch (org_glizy_config::get('PSW_METHOD'))
	{
		case 'MD5':
			return md5($psw);
		default:
			return $psw;
	}
}

function glz_basename($name)
{
	return preg_replace('/.php/', '', basename($name));
}

function glz_exception_handler($errno, $errstr, $errfile, $errline)
{
	$errorlevel=error_reporting();
	if ($errorlevel&$errno && !($errno&E_STRICT))
	{
		glz_import('org.glizy.Exception');		
		if (($severity & error_reporting()) == $severity)
		{
			org_glizy_Exception::show($errno, $errstr, $errfile, $errline);
		}
	}
}

function glz_empty($value)
{
	if ( strpos( $value, '<img') !== false ) return false;
	$value = is_string($value) ? strip_tags($value) : $value;
	return empty($value);
}

function glz_localeDate2default( $value )
{
	$type = strlen( $value ) == 10 ? 'date' : 'datetime';
	$reg = __T( $type == 'date' ? 'GLZ_DATE_TOTIME_REGEXP' : 'GLZ_DATETIME_TOTIME_REGEXP' );
	if ( is_array( $reg ) && preg_match( $reg[0], $value ) )
	{
		$value = preg_replace( $reg[0], $reg[1], $value );
	}
	return $value;
}

function glz_defaultDate2locale( $format, $value )
{
	list( $d, $t ) = explode( ' ', $value );
	list( $y, $m, $day ) = explode( '-', $d );
	list( $hh, $mm, $ss ) = explode( ':', $t );
	return date( $format, mktime( intval( $hh ), intval( $mm ), intval( $ss ), intval( $m ), intval( $day ), intval( $y ) ) );
}
/*
	code borrowed By Stewart Rosenberger
	http://www.stewartspeak.com/headings/   
	convert embedded, javascript unicode characters into embedded HTML
	entities. (e.g. '%u2018' => '&#8216;'). returns the converted string.
*/

/*
APL Quote	"	&quot;	&#34;	&#x22;
Double Quote (left) 		&ldquo;	&#8220;	&#x201C;
Double Quote (right)		&rdquo;	&#8221;	&#x201D;
Single Quote (left)		&lsquo;	&#8216;	&#x2018;
Single Quote (right)		&rsquo;	&#8217;	&#x2019;
Prime	'	&prime;	&#8242;	&#x2032;
Double Prime	?	&Prime;	&#8243;	&#x2033;
Em Dash		&mdash;	&#8212;	&#x2013;
En Dash		&ndash;	&#8211;	&#x2013;
Minus	-	&minus;	&#8722;	&#x2212;
Multiplication Symbol		&times;	&#215;	&#xD7;
Division Symbol		&divide;	&#247;	&#xF7;
Ellipsis		&hellip;	&#8230;	&#x2026;
Copyright Symbol		&copy;	&#169;	&#xA9;
Trademark		&trade;	&#8482;	&#x2122;
Registered Trademark		&reg;	&#174;	&#xAE;

*/

function javascript_to_html($text)
{
	$matches = null ;
	
	preg_match_all('/%u([0-9A-F]{4})/i',$text,$matches) ;
	if(!empty($matches)) 
	{
		$convTable = array(
							'2026' => "",
							'201C' => "",
							'201D' => "",
							'2018' => "",
							'2019' => "",
							'2032' => "'",
							'2013' => "",
							'2212' => ""
						);
		for($i=0;$i<sizeof($matches[0]);$i++)
		{
			if (isset($convTable[$matches[1][$i]]))
			{
				$text = str_replace($matches[0][$i], $convTable[$matches[1][$i]], $text);
			}
			else
			{
				$text = str_replace($matches[0][$i], '&#'.hexdec($matches[1][$i]).';',$text);
			}
		}
	}

	preg_match_all('/\&#([0-9A-F]{4});/i',$text,$matches) ;
	if(!empty($matches)) 
	{
		$convTable = array(
							'8230' => "",
							'8220' => "",
							'8221' => "",
							'8216' => "",
							'8217' => "",
							'8242' => "'",
							'8211' => "",
							'8212' => "-",
							'8722' => "",
							'8364' => ""
						);
		for($i=0;$i<sizeof($matches[0]);$i++)
		{
			if (isset($convTable[$matches[1][$i]]))
			{
				$text = str_replace($matches[0][$i], $convTable[$matches[1][$i]], $text);
			}
		}
	}
	return $text;
}

function glz_makePass( $len = 7)
{
	$pass = "";
	$salt = "abchefghjkmnpqrstuvwxyz0123456789";
	srand((double)microtime()*1000000);
	$i = 0;
	while ($i <= $len )
	{
		$num = rand() % 33;
		$tmp = substr($salt, $num, 1);
		$pass = $pass . $tmp;
		$i++;
	}
	return $pass;
}

function glz_makeConfirmCode( $len=7, $id=NULL )
{
       $convTable = array( 0 => "f", 1 => "x", 2 => "r", 3 => "i", 4 => "d",5 => "g", 6 => "n", 7 => "k", 8 => "h", 9 => "o" );

       // creazione password normale
       $code = glz_makePass( $len );
       // codifica di cesare per id
       $s_id = sprintf("%d",$id);
       for( $i=0; $i<strlen($id); $i++ )
       {
               // accodiamo la conversione dell'id al codice
               $code .= $convTable[ $s_id{$i} ];
       }
       return $code;
}

function glz_strtrim($str, $maxlen=200, $elli='...', $maxoverflow=15)
{
	$str = strip_tags( $str );
	if ( strlen($str) > $maxlen)
	{
		$output = NULL;
		$body = explode(" ", $str);
		$body_count = count($body);

		$i=0;
		do {
			$output .= $body[$i]." ";
			$thisLen = strlen($output);
			$cycle = ($thisLen < $maxlen && $i < $body_count-1 && ($thisLen+strlen($body[$i+1])) < $maxlen+$maxoverflow?true:false);
			$i++;
		} while ($cycle);
		return $output.$elli;
	}
	else return $str;
}

function glz_hostName()
{
	return GLZ_HOST.'/';
}

function glz_sanitizeUrlTitle($title) {
	$title = strtolower($title);
	$title = preg_replace('/&.+?;/', '', $title); // kill entities
	$title = preg_replace('/[^a-z0-9 _-]/', '', $title);
	$title = preg_replace('/\s+/', ' ', $title);
	$title = str_replace(' ', '-', $title);
	$title = preg_replace('|-+|', '-', $title);
	$title = trim($title, '-');

	return $title;
}

if ( !function_exists( "stripos" ) ) 
{
	function stripos($str,$needle)
	{
		return strpos(strtolower($str),strtolower($needle));
	}
}
?>